# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Generate IDFs."""


def inputData(self, ScriptType: str = None):
    """Input data for IDF generation.

    :param self: Used as a method for class ``accim.sim.accim_Main.accimJob``
    :param ScriptType: Inherited from class ``accim.sim.accis.addAccis``
    """
    CS_CA_CM_list_dict = {
        '0 = ESP CTE': [['n/a'], ['n/a']],
        '1 = INT EN16798-1': [[1, 2, 3], [0, 1, 2, 3]],
        '2 = INT ASHRAE55': [[80, 90], [0, 1, 2, 3]],
        '3 = JPN Rijal': [[80, 90], [0, 1, 2, 3]],
        '4 = CHN GBT50785 Cold': [[1, 2], [3]],
        '5 = CHN GBT50785 HotMild': [[1, 2], [3]],
        '6 = CHN Yang': [[80, 90], [0, 1, 2, 3]],
        '7 = IND IMAC C NV': [[80, 85, 90], [0, 1, 2, 3]],
        '8 = IND IMAC C MM': [[80, 85, 90], [0, 1, 2, 3]],
        '9 = IND IMAC R 7DRM': [[80, 90], [0, 1, 2, 3]],
        '10 = IND IMAC R 30DRM': [[80, 90], [0, 1, 2, 3]],
        '11 = IND Dhaka': [[80, 90], [0, 1, 2, 3]],
        '12 = ROM Udrea': [[80, 90], [0, 1, 2, 3]],
        '13 = AUS Williamson': [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        '14 = AUS DeDear': [[80, 90], [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]],
        '15 = BRA Rupp NV': [[80, 90], [0, 1, 2, 3]],
        '16 = BRA Rupp AC': [[80, 90], [0, 1, 2, 3]],
        '17 = MEX Oropeza Arid': [[80, 90], [0, 1, 2, 3]],
        '18 = MEX Oropeza DryTropic': [[80, 90], [0, 1, 2, 3]],
        '19 = MEX Oropeza Temperate': [[80, 90], [0, 1, 2, 3]],
        '20 = MEX Oropeza HumTropic': [[80, 90], [0, 1, 2, 3]],
        '21 = CHL Perez-Fargallo': [[80, 90], [2, 3]],
        '22 = INT ISO7730': [[1, 2, 3], [0]],
        '99 = CUSTOM': [['n/a'], [3]],
    }

    CS_CA_CM_data_dict = {
        0: {
            'name': '0 = ESP CTE',
            'CAT':{
                'n/a': 'n/a'
            },
            'ComfMod': {
                'n/a': 'n/a'
            }
        },
        1: {
            'name': '1 = INT EN16798',
            'CAT': {
                1: 'EN16798 Category I',
                2: 'EN16798 Category II',
                3: 'EN16798 Category III',
            },
            'ComfMod': {
                0: 'EN16798 Static setpoints',
                1: 'EN16798 Adaptive setpoints when applicable, otherwise CTE',
                2: 'EN16798 Adaptive setpoints when applicable, otherwise EN16798 Static setpoints',
                3: 'EN16798 Adaptive setpoints when applicable, otherwise EN16798 Adaptive setpoints horizontally extended',
            }
        },
        2: {
            'name': '2 = INT ASHRAE55',
            'CAT': {
                80: 'ASHRAE 55 80% acceptability',
                90: 'ASHRAE 55 90% acceptability',
            },
            'ComfMod': {
                0: 'ISO 7730 Static setpoints',
                1: 'ASHRAE 55 Adaptive setpoints when applicable, otherwise CTE',
                2: 'ASHRAE 55 Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'ASHRAE 55 Adaptive setpoints when applicable, otherwise ASHRAE 55 Adaptive setpoints horizontally extended',
            }
        },
        3: {
            'name': '3 = JPN Rijal',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'COOLBIZ Static setpoints',
                1: 'Rijal Model Adaptive setpoints when applicable, otherwise COOLBIZ Static setpoints',
                2: 'Rijal Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Rijal Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        4: {
            'name': '4 = CHN GBT50785 Cold',
            'CAT': {
                1: '90% acceptability',
                2: '75-90% acceptability',
            },
            'ComfMod': {
                # 0: 'X Static setpoints',
                # 1: 'GBT50785 Cold Model Adaptive setpoints when applicable, otherwise X Static setpoints',
                # 2: 'GBT50785 Cold Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'GBT50785 Cold Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        5: {
            'name': '5 = CHN GBT50785 HotMild',
            'CAT': {
                1: '90% acceptability',
                2: '75-90% acceptability',
            },
            'ComfMod': {
                # 0: 'X Static setpoints',
                # 1: 'GBT50785 HotMild Model Adaptive setpoints when applicable, otherwise X Static setpoints',
                # 2: 'GBT50785 HotMild Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'GBT50785 HotMild Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        6: {
            'name': '6 = CHN Yang',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                # 0: 'X Static setpoints',
                # 1: 'Yang Model Adaptive setpoints when applicable, otherwise X Static setpoints',
                2: 'Yang Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Yang Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        7: {
            'name': '7 = IND IMAC C NV',
            'CAT': {
                80: '80% acceptability',
                85: '85% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Indian Building Code Static setpoints',
                1: 'IMAC C NV Model Adaptive setpoints when applicable, otherwise Indian Building Code Static setpoints',
                2: 'IMAC C NV Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'IMAC C NV Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        8: {
            'name': '8 = IND IMAC C MM',
            'CAT': {
                80: '80% acceptability',
                85: '85% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Indian Building Code Static setpoints',
                1: 'IMAC C MM Model Adaptive setpoints when applicable, otherwise Indian Building Code Static setpoints',
                2: 'IMAC C MM Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'IMAC C MM Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        9: {
            'name': '9 = IND IMAC R 7DRM',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Indian Building Code Static setpoints',
                1: 'IMAC R 7DRM Model Adaptive setpoints when applicable, otherwise Indian Building Code Static setpoints',
                2: 'IMAC R 7DRM Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'IMAC R 7DRM Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        10: {
            'name': '10 = IND IMAC R 30DRM',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Indian Building Code Static setpoints',
                1: 'IMAC R 30DRM Model Adaptive setpoints when applicable, otherwise Indian Building Code Static setpoints',
                2: 'IMAC R 30DRM Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'IMAC R 30DRM Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        11: {
            'name': '11 = IND Dhaka',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Indian Building Code Static setpoints',
                1: 'Dhaka Model Adaptive setpoints when applicable, otherwise Indian Building Code Static setpoints',
                2: 'Dhaka Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Dhaka Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        12: {
            'name': '12 = ROM Udrea',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Romanian Building Code Static setpoints',
                1: 'Udrea Model Adaptive setpoints when applicable, otherwise Romanian Building Code Static setpoints',
                2: 'Udrea Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Udrea Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        13: {
            'name': '13 = AUS Williamson',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0.1: 'Australian Building Code Static setpoints for climate zones 1, 2 and 3',
                0.2: 'Australian Building Code Static setpoints for climate zone 4',
                0.3: 'Australian Building Code Static setpoints for climate zone 5',
                0.4: 'Australian Building Code Static setpoints for climate zones 6 and 7',
                0.5: 'Australian Building Code Static setpoints for climate zone 8',
                1.1: 'Williamson Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zones 1, 2 and 3',
                1.2: 'Williamson Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 4',
                1.3: 'Williamson Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 5',
                1.4: 'Williamson Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zones 6 and 7',
                1.5: 'Williamson Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 8',
                2: 'Williamson Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Williamson Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        14: {
            'name': '14 = AUS DeDear',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0.1: 'Australian Building Code Static setpoints for climate zones 1, 2 and 3',
                0.2: 'Australian Building Code Static setpoints for climate zone 4',
                0.3: 'Australian Building Code Static setpoints for climate zone 5',
                0.4: 'Australian Building Code Static setpoints for climate zones 6 and 7',
                0.5: 'Australian Building Code Static setpoints for climate zone 8',
                1.1: 'DeDear Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zones 1, 2 and 3',
                1.2: 'DeDear Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 4',
                1.3: 'DeDear Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 5',
                1.4: 'DeDear Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zones 6 and 7',
                1.5: 'DeDear Model Adaptive setpoints when applicable, otherwise Australian Building Code Static setpoints for climate zone 8',
                2: 'DeDear Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'DeDear Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        15: {
            'name': '15 = BRA Rupp NV',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Brazilian Building Code Static setpoints (ABNT NBR 16401-2 Standard (2008))',
                1: 'Rupp NV Model Adaptive setpoints when applicable, otherwise Brazilian Building Code Static setpoints',
                2: 'Rupp NV Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Rupp NV Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        16: {
            'name': '16 = BRA Rupp AC',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Brazilian Building Code Static setpoints (ABNT NBR 16401-2 Standard (2008))',
                1: 'Rupp AC Model Adaptive setpoints when applicable, otherwise Brazilian Building Code Static setpoints',
                2: 'Rupp AC Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Rupp AC Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        17: {
            'name': '17 = MEX Oropeza Arid',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Relevant Static setpoints for Mexico: 20 for heating and 25 for cooling',
                1: 'Oropeza Arid Model Adaptive setpoints when applicable, otherwise Relevant Static setpoints for Mexico',
                2: 'Oropeza Arid Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Oropeza Arid Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        18: {
            'name': '18 = MEX Oropeza DryTropic',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Relevant Static setpoints for Mexico: 20 for heating and 25 for cooling',
                1: 'Oropeza DryTropic Model Adaptive setpoints when applicable, otherwise Relevant Static setpoints for Mexico',
                2: 'Oropeza DryTropic Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Oropeza DryTropic Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        19: {
            'name': '19 = MEX Oropeza Temperate',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Relevant Static setpoints for Mexico: 20 for heating and 25 for cooling',
                1: 'Oropeza Temperate Model Adaptive setpoints when applicable, otherwise Relevant Static setpoints for Mexico',
                2: 'Oropeza Temperate Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Oropeza Temperate Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        20: {
            'name': '20 = MEX Oropeza HumTropic',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                0: 'Relevant Static setpoints for Mexico: 20 for heating and 25 for cooling',
                1: 'Oropeza HumTropic Model Adaptive setpoints when applicable, otherwise Relevant Static setpoints for Mexico',
                2: 'Oropeza HumTropic Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Oropeza HumTropic Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        21: {
            'name': '21 = CHL Perez-Fargallo',
            'CAT': {
                80: '80% acceptability',
                90: '90% acceptability',
            },
            'ComfMod': {
                2: 'Perez-Fargallo Model Adaptive setpoints when applicable, otherwise ISO 7730 Static setpoints',
                3: 'Perez-Fargallo Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },
        22: {
            'name': '22 = INT ISO7730',
            'CAT': {
                1: 'Cat A: -0.2 < PMV < 0.2; PPD < 6%',
                2: 'Cat B: -0.5 < PMV < 0.5; PPD < 10%',
                3: 'Cat C: -0.7 < PMV < 0.7; PPD < 15%',
            },
            'ComfMod': {
                0: 'ISO 7730 Static setpoints',
            }
        },
        99: {
            'name': '99 = CUSTOM',
            'CAT': {
                'n/a': 'n/a'
            },
            'ComfMod': {
                3: 'Custom Model Adaptive setpoints when applicable, otherwise Adaptive setpoints horizontally extended',
            }
        },

    }

    print('The information you will be required to enter below will be used to generate the customised output IDFs:')
    fullComfStandList = list(range(len(CS_CA_CM_list_dict)-1))
    fullComfStandList.append(99)
    self.ComfStand_List = list(int(num) for num in input(
        'Enter the Comfort Standard numbers separated by space (\n'
        '0 = ESP CTE;\n'
        '1 = INT EN16798-1;\n'
        '2 = INT ASHRAE55;\n'
        '3 = JPN Rijal;\n'
        '4 = CHN GBT50785 Cold;\n'
        '5 = CHN GBT50785 HotMild;\n'
        '6 = CHN Yang;\n'
        '7 = IND IMAC C NV;\n'
        '8 = IND IMAC C MM;\n'
        '9 = IND IMAC R 7DRM;\n'
        '10 = IND IMAC R 30DRM;\n'
        '11 = IND Dhaka;\n'
        '12 = ROM Udrea;\n'
        '13 = AUS Williamson;\n'
        '14 = AUS DeDear;\n'
        '15 = BRA Rupp NV;\n'
        '16 = BRA Rupp AC;\n'
        '17 = MEX Oropeza Arid;\n'
        '18 = MEX Oropeza DryTropic;\n'
        '19 = MEX Oropeza Temperate;\n'
        '20 = MEX Oropeza HumTropic;\n'
        '21 = CHL Perez-Fargallo;\n'
        '22 = INT ISO7730;\n'
        '99 = CUSTOM;\n'
        'Please refer to the full list of setpoint temperatures at https://htmlpreview.github.io/?https://github.com/dsanchez-garcia/accim/blob/master/docs/source/html_files/full_setpoint_table.html\n'
        '): '
    ).split())
    while len(self.ComfStand_List) == 0 or not all(elem in fullComfStandList for elem in self.ComfStand_List):
        print('          Comfort Standard numbers are not correct. Please enter the numbers again.')
        self.ComfStand_List = list(
            int(num) for num in input("     Enter the Comfort Standard numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.ComfStand_List = list(
            int(num) for num in input("     Enter the Comfort Standard numbers separated by space: ").split())
        while len(self.ComfStand_List) == 0 or not all(elem in fullComfStandList for elem in self.ComfStand_List):
            print('          Comfort Standard numbers are not correct. Please enter the numbers again.')
            self.ComfStand_List = list(
                int(num) for num in input("     Enter the Comfort Standard numbers separated by space: ").split())
    if 99 in self.ComfStand_List:
        print('\nYou have requested the use of a custom comfort model. Please enter the following data necessary to build the custom comfort model:')

        self.CustAST_m = float(input('\nEnter the m coefficient (slope) of comfort model linear regression (mx+n) (any number greater than 0): '))
        while self.CustAST_m < 0:
            print(f'          The number you entered for CustAST_m is {self.CustAST_m}, which is smaller than 0, and that is not allowed. ')
            self.CustAST_m = float(input('          Enter the m coefficient (slope) of comfort model linear regression (mx+n) (any number greater than 0): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_m = float(input('          Enter the m coefficient (slope) of comfort model linear regression (mx+n) (any number greater than 0): '))

        self.CustAST_n = float(input('\nEnter the n coefficient of comfort model linear regression (mx+n): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_n = float(input('          Enter the n coefficient of comfort model linear regression (mx+n): '))

        self.CustAST_AHSToffset = float(input('\nEnter the offset from neutral temperature for the heating setpoint (value will be summed, therefore, it should be negative): '))
        while self.CustAST_AHSToffset > 0:
            print(f'          The number you entered for CustAST_AHSToffset is {self.CustAST_AHSToffset}, which is larger than 0, and that is not allowed. ')
            self.CustAST_AHSToffset = float(input('          Enter the offset from neutral temperature for the heating setpoint (value will be summed, therefore, it should be negative): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_AHSToffset = float(input('          Enter the offset from neutral temperature for the heating setpoint (value will be summed, therefore, it should be negative): '))

        self.CustAST_ACSToffset = float(input('\nEnter the offset from neutral temperature for the cooling setpoint (value will be summed, therefore, it should be positive): '))
        while self.CustAST_ACSToffset < 0:
            print(f'          The number you entered for CustAST_ACSToffset is {self.CustAST_ACSToffset}, which is smaller than 0, and that is not allowed. ')
            self.CustAST_ACSToffset = float(input('          Enter the offset from neutral temperature for the cooling setpoint (value will be summed, therefore, it should be positive): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_ACSToffset = float(input('          Enter the offset from neutral temperature for the cooling setpoint (value will be summed, therefore, it should be positive): '))

        self.CustAST_ACSTall = float(input('\nEnter the value for the cooling setpoint applicability lower limit (ACSTall): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_ACSTall = float(input('          Enter the value for the cooling setpoint applicability lower limit (ACSTall): '))

        self.CustAST_ACSTaul = float(input('\nEnter the value for the cooling setpoint applicability upper limit (ACSTaul): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_ACSTaul = float(input('          Enter the value for the cooling setpoint applicability upper limit (ACSTaul): '))

        self.CustAST_AHSTall = float(
            input('\nEnter the value for the heating setpoint applicability lower limit (ACSTall): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_AHSTall = float(input('          Enter the value for the heating setpoint applicability lower limit (AHSTall): '))

        self.CustAST_AHSTaul = float(
            input('\nEnter the value for the heating setpoint applicability upper limit (ACSTall): '))
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CustAST_AHSTaul = float(input('          Enter the value for the heating setpoint applicability upper limit (AHSTaul): '))
    else:
        self.CustAST_m = 0
        self.CustAST_n = 0
        self.CustAST_AHSToffset = 0
        self.CustAST_ACSToffset = 0
        self.CustAST_ACSTall = 0
        self.CustAST_ACSTaul = 0
        self.CustAST_AHSTall = 0
        self.CustAST_AHSTaul = 0

    print('\n')
    for i in self.ComfStand_List:
        print('For the comfort standard ' + CS_CA_CM_data_dict[i]['name'] + ', the available categories you can choose are: ')
        for j in CS_CA_CM_data_dict[i]['CAT']:
            print(str(j) + ' = ' + CS_CA_CM_data_dict[i]['CAT'][j])

    fullCATlist = [1, 2, 3, 80, 85, 90]
    availableCATlist = []
    for i in self.ComfStand_List:
        availableCATlist.extend([j for j in CS_CA_CM_data_dict[i]['CAT'].keys() if j != 'n/a'])
    print("Enter the Category numbers separated by space (")
    if 1 in availableCATlist:
        print("1 = CAT I / CAT A;")
    if 2 in availableCATlist:
        print("2 = CAT II / CAT B;")
    if 3 in availableCATlist:
        print("3 = CAT III / CAT C;")
    if 80 in availableCATlist:
        print("80 = 80% ACCEPT;")
    if 85 in availableCATlist:
        print("85 = 85% ACCEPT;")
    if 90 in availableCATlist:
        print("90 = 90% ACCEPT;")
    print("Please refer to the full list of setpoint temperatures at https://htmlpreview.github.io/?https://github.com/dsanchez-garcia/accim/blob/master/docs/source/html_files/full_setpoint_table.html")
    self.CAT_List = list(int(num) for num in input('):').split())
    while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
        print('          Category numbers are not correct. Please enter the numbers again.')
        self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.CAT_List = list(int(num) for num in input("     Enter the Category numbers separated by space: ").split())
        while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
            print('          Category numbers are not correct. Please enter the numbers again.')
            self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())
    print('\n')
    for i in self.ComfStand_List:
        print('For the comfort standard ' + CS_CA_CM_data_dict[i]['name'] + ', the available ComfMods you can choose are: ')
        for j in CS_CA_CM_data_dict[i]['ComfMod']:
            print(str(j) + ' = ' + CS_CA_CM_data_dict[i]['ComfMod'][j])

    fullComfModList = [0, 0.1, 0.2, 0.3, 0.4, 0.5, 1, 1.1, 1.2, 1.3, 1.4, 1.5, 2, 3]
    self.ComfMod_List = list(float(num) for num in input(
        "Enter the Comfort Mode numbers separated by space (\n"
        "0 or 0.X = Static;\n"
        "1, 1.X, 2, 3 = Adaptive;\n"
        "Please refer to the full list of setpoint temperatures at https://htmlpreview.github.io/?https://github.com/dsanchez-garcia/accim/blob/master/docs/source/html_files/full_setpoint_table.html\n"
        "): ").split())
    while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
        print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
        self.ComfMod_List = list(
            float(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.ComfMod_List = list(
            float(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
        while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
            print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
            self.ComfMod_List = list(
                float(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
    try:
        self.SetpointAcc = float(input('\nEnter the setpoint accuracy number (any number greater than 0, if omitted will be 100): '))
    except ValueError:
        self.SetpointAcc = 100
    while self.SetpointAcc < 0:
        print('          The setpoint accuracy number is not correct. It must be a number greater than 0. Please enter the number again.')
        self.SetpointAcc = float(input('         Enter the setpoint accuracy number (any number greater than 0, if omitted will be 100): '))
    while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
        try:
            self.SetpointAcc = float(input('      Enter the setpoint accuracy number (any number greater than 0, if omitted will be 100): '))
        except ValueError:
            self.SetpointAcc = 100
        while self.SetpointAcc < 0:
            print('          The setpoint accuracy number is not correct. It must be a number greater than 0. Please enter the number again.')
            try:
                self.SetpointAcc = float(input('      Enter the setpoint accuracy number (any number greater than 0, if omitted will be 100): '))
            except ValueError:
                self.SetpointAcc = 100
    try:
        self.CATcoolOffset = float(input('\nEnter the number for the CAT cooling offset modifier (value will be summed to the ACST, if omitted will be 0): '))
    except ValueError:
        self.CATcoolOffset = 0.0
    while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
        try:
            self.CATcoolOffset = float(input('\n        Enter the number for the CAT cooling offset modifier (value will be summed to the ACST, if omitted will be 0): '))
        except ValueError:
            self.CATcoolOffset = 0.0

    try:
        self.CATheatOffset = float(input('\nEnter the number for the CAT heating offset modifier (value will be summed to the AHST, if omitted will be 0): '))
    except ValueError:
        self.CATheatOffset = 0.0
    while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
        try:
            self.CATheatOffset = float(input('\n        Enter the number for the CAT heating offset modifier (value will be summed to the AHST, if omitted will be 0): '))
        except ValueError:
            self.CATheatOffset = 0.0
    while self.CATheatOffset > self.CATcoolOffset:
        print(f'          You have entered a CATheatOffset ({self.CATheatOffset}) larger than the CATcoolOffset ({self.CATcoolOffset}), '
              f'which will probably lead to an error in the EnergyPlus simulation.')
        try:
            self.CATheatOffset = float(input('\n        Enter the number for the CAT heating offset modifier (value will be summed to the AHST, if omitted will be 0): '))
        except ValueError:
            self.CATheatOffset = 0.0
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            try:
                self.CATheatOffset = float(input('\n        Enter the number for the CAT heating offset modifier (value will be summed to the AHST, if omitted will be 0): '))
            except ValueError:
                self.CATheatOffset = 0.0
        # self.CATheatOffset = float(input('\nEnter the number for the CAT heating offset modifier (value will be summed to the AHST): '))
        # while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
        #     self.CATheatOffset = float(input('      Enter the number for the CAT heating offset modifier (value will be summed to the AHST): '))

    if (any(i in [1, 2] for i in self.ComfStand_List) and 0 in self.ComfMod_List) or 22 in self.ComfStand_List:
        self.CoolSeasonStart = list(
            int(num)
            for num
            in input("\nEnter the start of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
        )
        if len(self.CoolSeasonStart) == 1:
            day_of_year = self.CoolSeasonStart[0]
        elif len(self.CoolSeasonStart) == 2:
            from datetime import date
            day_of_year = date(2007, self.CoolSeasonStart[1], self.CoolSeasonStart[0]).timetuple().tm_yday
        while day_of_year < 1 or day_of_year > 365:
            print('          The start for cooling season is not correct. It must be a numeric date format dd/mm or the day of the year. Please enter the value again.')
            self.CoolSeasonStart = list(
                int(num)
                for num
                in input("Enter the start of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
            )
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CoolSeasonStart = list(
                int(num)
                for num
                in input("Enter the start of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
            )
            if len(self.CoolSeasonStart) == 1:
                day_of_year = self.CoolSeasonStart[0]
            elif len(self.CoolSeasonStart) == 2:
                day_of_year = date(2007, self.CoolSeasonStart[1], self.CoolSeasonStart[0]).timetuple().tm_yday
            while day_of_year < 1 or day_of_year > 365:
                print('          The start for cooling season is not correct. It must be a numeric date format dd/mm or the day of the year. Please enter the value again.')
                self.CoolSeasonStart = list(
                    int(num)
                    for num
                    in input("Enter the start of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
                )
                if len(self.CoolSeasonStart) == 1:
                    day_of_year = self.CoolSeasonStart[0]
                elif len(self.CoolSeasonStart) == 2:
                    day_of_year = date(2007, self.CoolSeasonStart[1], self.CoolSeasonStart[0]).timetuple().tm_yday
        self.CoolSeasonStart = day_of_year

        self.CoolSeasonEnd = list(
            int(num)
            for num
            in input("\nEnter the end of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
        )
        if len(self.CoolSeasonEnd) == 1:
            day_of_year = self.CoolSeasonEnd[0]
        elif len(self.CoolSeasonEnd) == 2:
            from datetime import date
            day_of_year = date(2007, self.CoolSeasonEnd[1], self.CoolSeasonEnd[0]).timetuple().tm_yday
        while day_of_year < 1 or day_of_year > 365:
            print('          The end for cooling season is not correct. It must be a numeric date format dd/mm or the day of the year. Please enter the value again.')
            self.CoolSeasonEnd = list(
                int(num)
                for num
                in input("Enter the end of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
            )
        while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
            self.CoolSeasonEnd = list(
                int(num)
                for num
                in input("Enter the end of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
            )
            if len(self.CoolSeasonEnd) == 1:
                day_of_year = self.CoolSeasonEnd[0]
            elif len(self.CoolSeasonEnd) == 2:
                day_of_year = date(2007, self.CoolSeasonEnd[1], self.CoolSeasonEnd[0]).timetuple().tm_yday
            while day_of_year < 1 or day_of_year > 365:
                print('          The end for cooling season is not correct. It must be a numeric date format dd/mm or the day of the year. Please enter the value again.')
                self.CoolSeasonEnd = list(
                    int(num)
                    for num
                    in input("Enter the end of the cooling season in numeric date format dd/mm or the day of the year: ").split('/')
                )
                if len(self.CoolSeasonEnd) == 1:
                    day_of_year = self.CoolSeasonEnd[0]
                elif len(self.CoolSeasonEnd) == 2:
                    day_of_year = date(2007, self.CoolSeasonEnd[1], self.CoolSeasonEnd[0]).timetuple().tm_yday
        self.CoolSeasonEnd = day_of_year
    else:
        self.CoolSeasonStart = 121
        self.CoolSeasonEnd = 274

    if 'mm' in ScriptType.lower():
        fullHVACmodeList = [0, 1, 2]
        self.HVACmode_List = list(int(num) for num in input(
            "\nEnter the HVAC Mode numbers separated by space (\n"
            "0 = Fully Air-conditioned;\n"
            "1 = Naturally ventilated;\n"
            "2 = Mixed Mode;\n"
            "): ").split())
        while len(self.HVACmode_List) == 0 or not all(elem in fullHVACmodeList for elem in self.HVACmode_List):
            print('          HVACmode numbers are not correct. Please enter the numbers again.')
            self.HVACmode_List = list(
                int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())
        while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
            self.HVACmode_List = list(
                int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())
            while len(self.HVACmode_List) == 0 or not all(elem in fullHVACmodeList for elem in self.HVACmode_List):
                print('          HVACmode numbers are not correct. Please enter the numbers again.')
                self.HVACmode_List = list(
                    int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())

        fullVentCtrlList = [0, 1, 2, 3]
        self.VentCtrl_List = list(int(num) for num in input(
            "\nEnter the Ventilation Control numbers separated by space (\n"
            "If HVACmode = 1:\n"
            "   0 = Ventilates above neutral temperature;\n"
            "   1 = Ventilates above upper comfort limit;\n"
            "If HVACmode = 2:\n"
            "   0 = Ventilates above neutral temperature and fully opens doors and windows;\n"
            "   1 = Ventilates above lower comfort limit and fully opens doors and windows;\n"
            "   2 = Ventilates above neutral temperature and opens doors and windows based on the customised venting opening factor;\n"
            "   3 = Ventilates above lower comfort limit and opens doors and windows based on the customised venting opening factor;\n"
            "): ").split())
        while len(self.VentCtrl_List) == 0 or not all(elem in fullVentCtrlList for elem in self.VentCtrl_List):
            print('          Ventilation Control numbers are not correct. Please enter the numbers again.')
            self.VentCtrl_List = list(
                int(num) for num in input("     Enter the Ventilation Control numbers separated by space: ").split())
        while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
            self.VentCtrl_List = list(
                int(num) for num in input("     Enter the Ventilation Control numbers separated by space: ").split())
            while len(self.VentCtrl_List) == 0 or not all(elem in fullVentCtrlList for elem in self.VentCtrl_List):
                print('          Ventilation Control numbers are not correct. Please enter the numbers again.')
                self.VentCtrl_List = list(int(num) for num in input(
                    "     Enter the Ventilation Control numbers separated by space: ").split())

        if any([i in self.VentCtrl_List for i in [2, 3]]):
            self.MaxTempDiffVOF = float(input('Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
            while self.MaxTempDiffVOF <= 0:
                print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
                self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
                self.MaxTempDiffVOF = float(input('      Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
                while self.MaxTempDiffVOF <= 0:
                    print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
                    self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))

            self.MinTempDiffVOF = float(input('Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
            while self.MinTempDiffVOF <= 0:
                print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
                self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
                self.MinTempDiffVOF = float(input('      Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
                while self.MinTempDiffVOF <= 0:
                    print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
                    self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))

            while self.MinTempDiffVOF >= self.MaxTempDiffVOF:
                print('The minimum temperature difference number you entered is larger than or equal to the maximum temperature difference number. Please enter both maximum and minimum temperature difference numbers again.')
                self.MaxTempDiffVOF = float(input('Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
                while self.MaxTempDiffVOF <= 0:
                    print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
                    self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
                while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
                    self.MaxTempDiffVOF = float(input('      Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
                    while self.MaxTempDiffVOF <= 0:
                        print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
                        self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))

                self.MinTempDiffVOF = float(input('Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
                while self.MinTempDiffVOF <= 0:
                    print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
                    self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
                while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
                    self.MinTempDiffVOF = float(input('      Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
                    while self.MinTempDiffVOF <= 0:
                        print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
                        self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))

            self.MultiplierVOF = float(input('Enter the multiplier number for Ventilation Opening Factor (any number between 0 and 1): '))
            while self.MultiplierVOF < 0 or self.MultiplierVOF > 1:
                print('          The multiplier number is not correct. It must be a number between 0 and 1. Please enter the number again.')
                self.MultiplierVOF = float(input('         Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
                self.MultiplierVOF = float(input('      Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
                while self.MultiplierVOF < 0 or self.MultiplierVOF > 1:
                    print('          The multiplier number is not correct. It must be a number between 0 and 1. Please enter the number again.')
                    self.MultiplierVOF = float(input('         Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
        else:
            self.MaxTempDiffVOF = 6
            self.MinTempDiffVOF = 1
            self.MultiplierVOF = 0.25

        self.VSToffset_List = list(float(num) for num in input(
            "\nEnter the VSToffset numbers separated by space (if omitted, will be 0): ").split())
        if len(self.VSToffset_List) == 0:
            self.VSToffset_List = [float(0)]
        while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
            self.VSToffset_List = list(float(num) for num in input(
                "     Enter the VSToffset numbers separated by space (if omitted, will be 0): ").split())
            if len(self.VSToffset_List) == 0:
                self.VSToffset_List = [float(0)]

        self.MinOToffset_List = list(float(num) for num in input(
            "\nEnter the MinOToffset numbers separated by space (if omitted, will be 50): ").split())
        if len(self.MinOToffset_List) == 0:
            self.MinOToffset_List = [float(50)]
        while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
            self.MinOToffset_List = list(float(num) for num in input(
                "     Enter the MinOToffset numbers separated by space (if omitted, will be 50): ").split())
            if len(self.MinOToffset_List) == 0:
                self.MinOToffset_List = [float(50)]

        self.MaxWindSpeed_List = list(float(num) for num in input(
            "\nEnter the MaxWindSpeed numbers separated by space (if omitted, will be 50): ").split())
        if len(self.MaxWindSpeed_List) == 0:
            self.MaxWindSpeed_List = [float(50)]
        while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
            self.MaxWindSpeed_List = list(float(num) for num in input(
                "     Enter the MaxWindSpeed numbers separated by space (if omitted, will be 50): ").split())
            if len(self.MaxWindSpeed_List) == 0:
                self.MaxWindSpeed_List = [float(50)]
    elif 'ac' in ScriptType.lower():
        self.HVACmode_List = [0]
        self.VentCtrl_List = [0]
        self.MaxTempDiffVOF = 1
        self.MinTempDiffVOF = 0
        self.MultiplierVOF = 0
        self.VSToffset_List = [0]
        self.MinOToffset_List = [0]
        self.MaxWindSpeed_List = [0]

    try:
        self.ASTtol_value_from = float(input('\nEnter the ASTtol value from (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_from = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_from = float(input('     Enter the ASTtol value from (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_from = float(0.1)

    try:
        self.ASTtol_value_to_input = float(input('Enter the ASTtol value to (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_to_input = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_to_input = float(input('     Enter the ASTtol value to (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_to_input = float(0.1)

    try:
        self.ASTtol_value_steps = float(input('Enter the ASTtol value steps (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_steps = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_steps = float(input('     Enter the ASTtol value steps (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_steps = float(0.1)

    self.user_input_arguments = {
        'ComfStand': self.ComfStand_List,
        'CAT': self.CAT_List,
        'CATcoolOffset': self.CATcoolOffset,
        'CATheatOffset': self.CATheatOffset,
        'ComfMod': self.ComfMod_List,
        'SetpointAcc': self.SetpointAcc,
        'CustAST_ACSTaul': self.CustAST_ACSTaul,
        'CustAST_ACSTall': self.CustAST_ACSTall,
        'CustAST_AHSTaul': self.CustAST_AHSTaul,
        'CustAST_AHSTall': self.CustAST_AHSTall,
        'CustAST_m': self.CustAST_m,
        'CustAST_n': self.CustAST_n,
        'CustAST_ACSToffset': self.CustAST_ACSToffset,
        'CustAST_AHSToffset': self.CustAST_AHSToffset,
        'CoolSeasonStart': self.CoolSeasonStart,
        'CoolSeasonEnd': self.CoolSeasonEnd,
        'HVACmode': self.HVACmode_List,
        'VentCtrl': self.VentCtrl_List,
        'MaxTempDiffVOF': self.MaxTempDiffVOF,
        'MinTempDiffVOF': self.MinTempDiffVOF,
        'MultiplierVOF': self.MultiplierVOF,
        'VSToffset': self.VSToffset_List,
        'MinOToffset': self.MinOToffset_List,
        'MaxWindSpeed': self.MaxWindSpeed_List,
        'ASTtol_start': self.ASTtol_value_from,
        'ASTtol_end_input': self.ASTtol_value_to_input,
        'ASTtol_steps': self.ASTtol_value_steps,
    }


def genIDF(self,
    ScriptType: str = None,
    TempCtrl: str = None,
    ComfStand: list = None,
    CAT: list = None,
    CATcoolOffset: float = 0,
    CATheatOffset: float = 0,
    ComfMod: list = None,
    SetpointAcc: float = 10000,
    CustAST_ACSTaul: float = 0,
    CustAST_ACSTall: float = 0,
    CustAST_AHSTaul: float = 0,
    CustAST_AHSTall: float = 0,
    CustAST_m: float = 0,
    CustAST_n: float = 0,
    CustAST_ACSToffset: float = 0,
    CustAST_AHSToffset: float = 0,
    CoolSeasonStart=121,
    CoolSeasonEnd=274,
    HVACmode: list = None,
    VentCtrl: list = None,
    MaxTempDiffVOF: float = 6,
    MinTempDiffVOF: float = 1,
    MultiplierVOF: float = 0.25,
    VSToffset: list = [0],
    MinOToffset: list = [50],
    MaxWindSpeed: list = [50],
    ASTtol_start: float = 0.1,
    ASTtol_end_input: float = 0.1,
    ASTtol_steps: float = 0.1,
    NameSuffix: str = '',
    verboseMode: bool = True,
    confirmGen: bool = None
):
    """Generate IDFs.

    :param self: Used as a method for class ``accim.sim.accim_Main.accimJob``
    :param ScriptType: Inherited from class ``accim.sim.accis.addAccis``
    :param TempCtrl: Inherited from class ``accim.sim.accis.addAccis``
    :param ComfStand: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_m: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_n: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_ACSToffset: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_AHSToffset: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_ACSTall: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_ACSTaul: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_AHSTall: Inherited from class ``accim.sim.accis.addAccis``
    :param CustAST_AHSTaul: Inherited from class ``accim.sim.accis.addAccis``
    :param CAT: Inherited from class ``accim.sim.accis.addAccis``
    :param CATcoolOffset: Inherited from class ``accim.sim.accis.addAccis``
    :param CATheatOffset: Inherited from class ``accim.sim.accis.addAccis``
    :param ComfMod: Inherited from :class:``accim.sim.accis.addAccis``
    :param SetpointAcc: Inherited from :class:``accim.sim.accis.addAccis``
    :param CoolSeasonStart: Inherited from :class:``accim.sim.accis.addAccis``
    :param CoolSeasonEnd: Inherited from :class:``accim.sim.accis.addAccis``
    :param HVACmode: Inherited from :class:``accim.sim.accis.addAccis``
    :param VentCtrl: Inherited from :class:``accim.sim.accis.addAccis``
    :param MaxTempDiffVOF: Inherited from :class:``accim.sim.accis.addAccis``
    :param MinTempDiffVOF: Inherited from :class:``accim.sim.accis.addAccis``
    :param MultiplierVOF: Inherited from :class:``accim.sim.accis.addAccis``
    :param VSToffset: Inherited from :class:``accim.sim.accis.addAccis``
    :param MinOToffset: Inherited from :class:``accim.sim.accis.addAccis``
    :param MaxWindSpeed: Inherited from :class:``accim.sim.accis.addAccis``
    :param ASTtol_start: Inherited from :class:``accim.sim.accis.addAccis``
    :param ASTtol_end_input: Inherited from :class:``accim.sim.accis.addAccis``
    :param ASTtol_steps: Inherited from :class:``accim.sim.accis.addAccis``
    :param NameSuffix: Inherited from :class:``accim.sim.accis.addAccis``
    :param verboseMode: Inherited from :class:``accim.sim.accis.addAccis``
    :param confirmGen: Inherited from :class:``accim.sim.accis.addAccis``
    """
    import os
    from os import listdir
    import numpy
    from eppy import modeleditor
    from eppy.modeleditor import IDF
    from besos.eppy_funcs import get_building
    # import time
    # from tqdm import tqdm

    arguments = (ComfStand is None,
                 CAT is None,
                 CATcoolOffset == 0,
                 CATheatOffset == 0,
                 ComfMod is None,
                 SetpointAcc == 10000,
                 CustAST_ACSTaul == 0,
                 CustAST_ACSTall == 0,
                 CustAST_AHSTaul == 0,
                 CustAST_AHSTall == 0,
                 CustAST_m == 0,
                 CustAST_n == 0,
                 CustAST_ACSToffset == 0,
                 CustAST_AHSToffset == 0,
                 CoolSeasonStart == 121,
                 CoolSeasonEnd == 274,
                 HVACmode is None,
                 VentCtrl is None,
                 MaxTempDiffVOF == 6,
                 MinTempDiffVOF == 1,
                 MultiplierVOF == 0.25,
                 VSToffset == [0],
                 MinOToffset == [50],
                 MaxWindSpeed == [50],
                 ASTtol_start == 0.1,
                 ASTtol_end_input == 0.1,
                 ASTtol_steps == 0.1)
    if all(arguments):
        self.ASTtol_value_to = self.ASTtol_value_to_input + self.ASTtol_value_steps
    else:
        ASTtol_end = ASTtol_end_input + ASTtol_steps

    if all(arguments):
        self.ASTtol_value_from = round(self.ASTtol_value_from, 2)
        self.ASTtol_value_to = round(self.ASTtol_value_to, 2)
        self.ASTtol_value_steps = round(self.ASTtol_value_steps, 2)
    else:
        self.ComfStand_List = ComfStand
        self.CAT_List = CAT
        self.ComfMod_List = ComfMod
        self.SetpointAcc = SetpointAcc
        self.CATcoolOffset = CATcoolOffset
        self.CATheatOffset = CATheatOffset
        self.CustAST_ACSTaul = CustAST_ACSTaul
        self.CustAST_ACSTall = CustAST_ACSTall
        self.CustAST_AHSTaul = CustAST_AHSTaul
        self.CustAST_AHSTall = CustAST_AHSTall
        self.CustAST_m = CustAST_m
        self.CustAST_n = CustAST_n
        self.CustAST_ACSToffset = CustAST_ACSToffset
        self.CustAST_AHSToffset = CustAST_AHSToffset

        
        if type(CoolSeasonStart) is str:
            CoolSeasonStart = list(int(num) for num in CoolSeasonStart.split('/'))
            from datetime import date
            day_of_year = date(2007, CoolSeasonStart[1], CoolSeasonStart[0]).timetuple().tm_yday
        elif type(CoolSeasonStart) is int:
            day_of_year = CoolSeasonStart
        self.CoolSeasonStart = day_of_year
        
        # CoolSeasonEnd = list(int(num) for num in CoolSeasonEnd.split('/'))
        # if len(CoolSeasonEnd) == 1:
        #     day_of_year = CoolSeasonEnd[0]
        # elif len(CoolSeasonEnd) == 2:
        #     from datetime import date
        #     day_of_year = date(2007, CoolSeasonEnd[1], CoolSeasonEnd[0]).timetuple().tm_yday
        # self.CoolSeasonEnd = day_of_year
        if type(CoolSeasonEnd) is str:
            CoolSeasonEnd = list(int(num) for num in CoolSeasonEnd.split('/'))
            from datetime import date
            day_of_year = date(2007, CoolSeasonEnd[1], CoolSeasonEnd[0]).timetuple().tm_yday
        elif type(CoolSeasonEnd) is int:
            day_of_year = CoolSeasonEnd
        self.CoolSeasonEnd = day_of_year

        
        self.HVACmode_List = HVACmode
        self.VentCtrl_List = VentCtrl
        self.MaxTempDiffVOF = MaxTempDiffVOF,
        self.MinTempDiffVOF = MinTempDiffVOF,
        self.MultiplierVOF = MultiplierVOF,
        self.VSToffset_List = VSToffset
        self.MinOToffset_List = MinOToffset
        self.MaxWindSpeed_List = MaxWindSpeed
        self.ASTtol_value_from = round(ASTtol_start, 2)
        self.ASTtol_value_to = round(ASTtol_end, 2)
        self.ASTtol_value_steps = round(ASTtol_steps, 2)

    if 'ac' in ScriptType.lower():
        self.HVACmode_List = [0]
        self.VentCtrl_List = [0]
        self.MaxTempDiffVOF = 1,
        self.MinTempDiffVOF = 0,
        self.MultiplierVOF = 0,
        self.VSToffset_List = [0]
        self.MinOToffset_List = [0]
        self.MaxWindSpeed_List = [0]

    if any([i in self.VentCtrl_List for i in [2, 3]]):
        if type(self.MaxTempDiffVOF) is tuple:
            self.MaxTempDiffVOF = self.MaxTempDiffVOF[0]
        if type(self.MinTempDiffVOF) is tuple:
            self.MinTempDiffVOF = self.MinTempDiffVOF[0]
        if type(self.MultiplierVOF) is tuple:
            self.MultiplierVOF = self.MultiplierVOF[0]

    self.VSToffset_List = [float(i) for i in self.VSToffset_List]
    self.MinOToffset_List = [float(i) for i in self.MinOToffset_List]
    self.MaxWindSpeed_List = [float(i) for i in self.MaxWindSpeed_List]

    if NameSuffix == '':
        suffix = '[NS_X'
    else:
        suffix = '[NS_' + NameSuffix

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    filelist_pymod = ([file.split('.idf')[0] for file in filelist_pymod])
    # print(filelist_pymod)

    # Characters not admitted: & ^ , = % " / \ : * ? " < > |
    ComfStand_dict = {
        0: '[CS_ESP CTE',
        1: '[CS_INT EN16798',
        2: '[CS_INT ASHRAE55',
        3: '[CS_JPN Rijal',
        4: '[CS_CHN GBT50785 Cold',
        5: '[CS_CHN GBT50785 HotMild',
        6: '[CS_CHN Yang',
        7: '[CS_IND IMAC C NV',
        8: '[CS_IND IMAC C MM',
        9: '[CS_IND IMAC R 7DRM',
        10: '[CS_IND IMAC R 30DRM',
        11: '[CS_IND Dhaka',
        12: '[CS_ROU Udrea',
        13: '[CS_AUS Williamson',
        14: '[CS_AUS DeDear',
        15: '[CS_BRA Rupp NV',
        16: '[CS_BRA Rupp AC',
        17: '[CS_MEX Oropeza Arid',
        18: '[CS_MEX Oropeza DryTropic',
        19: '[CS_MEX Oropeza Temperate',
        20: '[CS_MEX Oropeza HumTropic',
        21: '[CS_CHL Perez-Fargallo',
        22: '[CS_INT ISO7730',
        99: '[CS_CUSTOM',
    }

    outputlist = []
    for file in filelist_pymod:
        filename = file.replace('_pymod', '')
        if TempCtrl.lower() == 'temp' or TempCtrl.lower() == 'temperature':
            for ComfStand_value in self.ComfStand_List:
                if ComfStand_value == 0:
                    for HVACmode_value in self.HVACmode_List:
                        if HVACmode_value == 0:
                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to,
                                                             self.ASTtol_value_steps):
                                outputname = (
                                        filename
                                        + ComfStand_dict[ComfStand_value]
                                        + '[CA_X'
                                        + '[CM_X'
                                        + '[HM_' + str(HVACmode_value)
                                        + '[VC_X'
                                        + '[VO_X'
                                        + '[MT_X'
                                        + '[MW_X'
                                        + '[AT_' + str(round(ASTtol_value, 2))
                                        + suffix
                                        + '.idf'
                                )
                                outputlist.append(outputname)
                        else:
                            for VentCtrl_value in self.VentCtrl_List:
                                if HVACmode_value == 1 and VentCtrl_value == 2:
                                    continue
                                elif HVACmode_value == 1 and VentCtrl_value == 3:
                                    continue
                                else:
                                    for VSToffset_value in self.VSToffset_List:
                                        for MinOToffset_value in self.MinOToffset_List:
                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                 self.ASTtol_value_to,
                                                                                 self.ASTtol_value_steps):
                                                    outputname = (
                                                            filename
                                                            + ComfStand_dict[ComfStand_value]
                                                            + '[CA_X'
                                                            + '[CM_X'
                                                            + '[HM_' + str(HVACmode_value)
                                                            + '[VC_' + str(VentCtrl_value)
                                                            + '[VO_' + str(VSToffset_value)
                                                            + '[MT_' + str(MinOToffset_value)
                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                            + suffix
                                                            + '.idf'
                                                    )
                                                    outputlist.append(outputname)
                elif ComfStand_value in [1, 4, 5, 22]:
                    for CAT_value in self.CAT_List:
                        if ComfStand_value in [1, 22] and CAT_value not in range(0, 4):
                            continue
                        elif ComfStand_value in [4, 5] and CAT_value not in [1, 2]:
                            continue
                        else:
                            for ComfMod_value in self.ComfMod_List:
                                if ComfStand_value not in [13, 14] and ComfMod_value in [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5]:
                                    continue
                                elif ComfStand_value == 22 and ComfMod_value != 0:
                                    continue
                                else:
                                    for HVACmode_value in self.HVACmode_List:
                                        if HVACmode_value == 0:
                                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to,
                                                                             self.ASTtol_value_steps):
                                                outputname = (
                                                        filename
                                                        + ComfStand_dict[ComfStand_value]
                                                        + '[CA_' + str(CAT_value)
                                                        + '[CM_' + str(ComfMod_value)
                                                        + '[HM_' + str(HVACmode_value)
                                                        + '[VC_X'
                                                        + '[VO_X'
                                                        + '[MT_X'
                                                        + '[MW_X'
                                                        + '[AT_' + str(round(ASTtol_value, 2))
                                                        + suffix
                                                        + '.idf'
                                                )
                                                outputlist.append(outputname)
                                        else:
                                            for VentCtrl_value in self.VentCtrl_List:
                                                if HVACmode_value == 1 and VentCtrl_value == 2:
                                                    continue
                                                elif HVACmode_value == 1 and VentCtrl_value == 3:
                                                    continue
                                                else:
                                                    for VSToffset_value in self.VSToffset_List:
                                                        for MinOToffset_value in self.MinOToffset_List:
                                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                                 self.ASTtol_value_to,
                                                                                                 self.ASTtol_value_steps):
                                                                    outputname = (
                                                                            filename
                                                                            + ComfStand_dict[ComfStand_value]
                                                                            + '[CA_' + str(CAT_value)
                                                                            + '[CM_' + str(ComfMod_value)
                                                                            + '[HM_' + str(HVACmode_value)
                                                                            + '[VC_' + str(VentCtrl_value)
                                                                            + '[VO_' + str(VSToffset_value)
                                                                            + '[MT_' + str(MinOToffset_value)
                                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                                            + suffix
                                                                            + '.idf'
                                                                    )
                                                                    outputlist.append(outputname)
                elif ComfStand_value in [2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 99]:
                    for CAT_value in self.CAT_List:
                        if ComfStand_value in [2, 3, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                                               20, 21] and CAT_value not in range(80, 91, 10):
                            continue
                        elif ComfStand_value in [7, 8] and CAT_value not in range(80, 91, 5):
                            continue
                        else:
                            for ComfMod_value in self.ComfMod_List:
                                if ComfStand_value in [13, 14] and ComfMod_value in [0, 1]:
                                    continue
                                elif ComfStand_value not in [13, 14] and ComfMod_value in [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5]:
                                    continue
                                elif ComfStand_value == 21 and ComfMod_value not in [2, 3]:
                                    continue
                                else:
                                    for HVACmode_value in self.HVACmode_List:
                                        if HVACmode_value == 0:
                                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to,
                                                                             self.ASTtol_value_steps):
                                                outputname = (
                                                        filename
                                                        + ComfStand_dict[ComfStand_value]
                                                        + '[CA_' + str(CAT_value)
                                                        + '[CM_' + str(ComfMod_value)
                                                        + '[HM_' + str(HVACmode_value)
                                                        + '[VC_X'
                                                        + '[VO_X'
                                                        + '[MT_X'
                                                        + '[MW_X'
                                                        + '[AT_' + str(round(ASTtol_value, 2))
                                                        + suffix
                                                        + '.idf'
                                                )
                                                outputlist.append(outputname)
                                        else:
                                            for VentCtrl_value in self.VentCtrl_List:
                                                if HVACmode_value == 1 and VentCtrl_value == 2:
                                                    continue
                                                elif HVACmode_value == 1 and VentCtrl_value == 3:
                                                    continue
                                                else:
                                                    for VSToffset_value in self.VSToffset_List:
                                                        for MinOToffset_value in self.MinOToffset_List:
                                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                                 self.ASTtol_value_to,
                                                                                                 self.ASTtol_value_steps):
                                                                    outputname = (
                                                                            filename
                                                                            + ComfStand_dict[ComfStand_value]
                                                                            + '[CA_' + str(CAT_value)
                                                                            + '[CM_' + str(ComfMod_value)
                                                                            + '[HM_' + str(HVACmode_value)
                                                                            + '[VC_' + str(VentCtrl_value)
                                                                            + '[VO_' + str(VSToffset_value)
                                                                            + '[MT_' + str(MinOToffset_value)
                                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                                            + suffix
                                                                            + '.idf'
                                                                    )
                                                                    outputlist.append(outputname)
        elif TempCtrl.lower() == 'pmv':
            outputname = (
                    filename
                    + '[CS_PMV'
                    + '[CA_X'
                    + '[CM_X'
                    + '[HM_X'
                    + '[VC_X'
                    + '[VO_X'
                    + '[MT_X'
                    + '[MW_X'
                    + '[AT_X'
                    + suffix
                    + '.idf'
            )
            outputlist.append(outputname)

    if verboseMode:
        print('The list of output IDFs is going to be:')
        print(*outputlist, sep="\n")
        print(f'And the total number of output IDFs is going to be {len(outputlist)}')

    if confirmGen is None:
        confirmGen = input('Do you still want to run ACCIS? [y/n]: ')
        if confirmGen == 'y':
            confirmGen = True
        else:
            confirmGen = False

    if confirmGen == True:
        if verboseMode:
            print('Generating the following output IDF files:')
        # pbar = tqdm(total=len(outputlist))
        # self.output_idf_dict = {}
        for file in filelist_pymod:
            filename = file

            fname1 = filename + '.idf'

            filename = file.replace('_pymod', '')
            # if verboseMode:
            #     print(f'Taking "{fname1}" as input IDF file:')

            # idf1 = IDF(fname1)
            idf1 = get_building(fname1)

            # print(filename)
            SetInputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                             program.Name == 'SetInputData'])
            ApplyCAT = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                             program.Name == 'ApplyCAT'])
            SetVOFinputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                             program.Name == 'SetVOFinputData'])
            SetAST = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                             program.Name == 'SetAST'])
            SetComfTemp= ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                       program.Name == 'SetComfTemp'])
            SetAppLimits= ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                       program.Name == 'SetAppLimits'])
            if TempCtrl.lower() == 'temp' or TempCtrl.lower() == 'temperature':
                for ComfStand_value in self.ComfStand_List:
                    SetInputData[0].Program_Line_1 = 'set ComfStand = ' + str(ComfStand_value)
                    if ComfStand_value == 0:
                        SetInputData[0].Program_Line_2 = 'set CAT = 1'
                        SetInputData[0].Program_Line_3 = 'set ComfMod = 0'
                        for HVACmode_value in self.HVACmode_List:
                            SetInputData[0].Program_Line_4 = 'set HVACmode = ' + str(HVACmode_value)
                            if HVACmode_value == 0:
                                for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to,
                                                                 self.ASTtol_value_steps):
                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)
                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                    outputname = (
                                            filename
                                            + ComfStand_dict[ComfStand_value]
                                            + '[CA_X'
                                            + '[CM_X'
                                            + '[HM_' + str(HVACmode_value)
                                            + '[VC_X'
                                            + '[VO_X'
                                            + '[MT_X'
                                            + '[MW_X'
                                            + '[AT_' + str(round(ASTtol_value, 2))
                                            + suffix
                                            + '.idf'
                                    )
                                    if verboseMode:
                                        print(outputname)
                                        # time.sleep(0.1)
                                        # pbar.update(1)
                                    idf1.savecopy(outputname)
                                    self.output_idf_dict.update({outputname: idf1})
                            else:
                                for VentCtrl_value in self.VentCtrl_List:
                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + str(VentCtrl_value)
                                    if HVACmode_value == 2:
                                        if VentCtrl_value == 2 or VentCtrl_value == 3:
                                            SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + str(self.MaxTempDiffVOF)
                                            SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + str(self.MinTempDiffVOF)
                                            SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + str(self.MultiplierVOF)
                                    for VSToffset_value in self.VSToffset_List:
                                        SetInputData[0].Program_Line_6 = 'set VSToffset = ' + str(VSToffset_value)
                                        for MinOToffset_value in self.MinOToffset_List:
                                            SetInputData[0].Program_Line_7 = 'set MinOToffset = ' + str(
                                                MinOToffset_value)
                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                SetInputData[0].Program_Line_8 = 'set MaxWindSpeed = ' + str(
                                                    MaxWindSpeed_value)
                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                 self.ASTtol_value_to,
                                                                                 self.ASTtol_value_steps):
                                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)
                                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                                    outputname = (
                                                            filename
                                                            + ComfStand_dict[ComfStand_value]
                                                            + '[CA_X'
                                                            + '[CM_X'
                                                            + '[HM_' + str(HVACmode_value)
                                                            + '[VC_' + str(VentCtrl_value)
                                                            + '[VO_' + str(VSToffset_value)
                                                            + '[MT_' + str(MinOToffset_value)
                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                            + suffix
                                                            + '.idf'
                                                    )
                                                    if verboseMode:
                                                        print(outputname)
                                                        # time.sleep(0.1)
                                                        # pbar.update(1)
                                                    idf1.savecopy(outputname)
                                                    self.output_idf_dict.update({outputname: idf1})
                    elif ComfStand_value in [1, 4, 5, 22]:
                        for CAT_value in self.CAT_List:
                            if ComfStand_value in [1, 22] and CAT_value not in range(0, 4):
                                continue
                            elif ComfStand_value in [4, 5] and CAT_value not in [1, 2]:
                                continue
                            else:
                                SetInputData[0].Program_Line_2 = 'set CAT = ' + str(CAT_value)
                                for ComfMod_value in self.ComfMod_List:
                                    if ComfStand_value not in [13, 14] and ComfMod_value in [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5]:
                                        continue
                                    elif ComfStand_value == 22 and ComfMod_value != 0:
                                        continue
                                    else:
                                        SetInputData[0].Program_Line_3 = 'set ComfMod = ' + str(ComfMod_value)
                                        for HVACmode_value in self.HVACmode_List:
                                            SetInputData[0].Program_Line_4 = 'set HVACmode = ' + str(HVACmode_value)
                                            if HVACmode_value == 0:
                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                 self.ASTtol_value_to,
                                                                                 self.ASTtol_value_steps):
                                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)
                                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                                    outputname = (
                                                            filename
                                                            + ComfStand_dict[ComfStand_value]
                                                            + '[CA_' + str(CAT_value)
                                                            + '[CM_' + str(ComfMod_value)
                                                            + '[HM_' + str(HVACmode_value)
                                                            + '[VC_X'
                                                            + '[VO_X'
                                                            + '[MT_X'
                                                            + '[MW_X'
                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                            + suffix
                                                            + '.idf'
                                                    )
                                                    if verboseMode:
                                                        print(outputname)
                                                        # time.sleep(0.1)
                                                        # pbar.update(1)
                                                    idf1.savecopy(outputname)
                                                    self.output_idf_dict.update({outputname: idf1})
                                            else:
                                                for VentCtrl_value in self.VentCtrl_List:
                                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + str(VentCtrl_value)
                                                    if HVACmode_value == 2:
                                                        if VentCtrl_value == 2 or VentCtrl_value == 3:
                                                            SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + str(self.MaxTempDiffVOF)
                                                            SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + str(self.MinTempDiffVOF)
                                                            SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + str(self.MultiplierVOF)
                                                    for VSToffset_value in self.VSToffset_List:
                                                        SetInputData[0].Program_Line_6 = 'set VSToffset = ' + str(
                                                            VSToffset_value)
                                                        for MinOToffset_value in self.MinOToffset_List:
                                                            SetInputData[0].Program_Line_7 = 'set MinOToffset = ' + str(
                                                                MinOToffset_value)
                                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                                SetInputData[
                                                                    0].Program_Line_8 = 'set MaxWindSpeed = ' + str(
                                                                    MaxWindSpeed_value)
                                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                                 self.ASTtol_value_to,
                                                                                                 self.ASTtol_value_steps):
                                                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)

                                                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                                                    outputname = (
                                                                            filename
                                                                            + ComfStand_dict[ComfStand_value]
                                                                            + '[CA_' + str(CAT_value)
                                                                            + '[CM_' + str(ComfMod_value)
                                                                            + '[HM_' + str(HVACmode_value)
                                                                            + '[VC_' + str(VentCtrl_value)
                                                                            + '[VO_' + str(VSToffset_value)
                                                                            + '[MT_' + str(MinOToffset_value)
                                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                                            + suffix
                                                                            + '.idf'
                                                                    )
                                                                    if verboseMode:
                                                                        print(outputname)
                                                                        # time.sleep(0.1)
                                                                        # pbar.update(1)
                                                                    idf1.savecopy(outputname)
                                                                    self.output_idf_dict.update({outputname: idf1})
                    elif ComfStand_value in [2, 3, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 99]:
                        for CAT_value in self.CAT_List:
                            if ComfStand_value in [2, 3, 6, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                                                   20, 21] and CAT_value not in range(80, 91, 10):
                                continue
                            elif ComfStand_value in [7, 8] and CAT_value not in range(80, 91, 5):
                                continue
                            else:
                                SetInputData[0].Program_Line_2 = 'set CAT = ' + str(CAT_value)
                                for ComfMod_value in self.ComfMod_List:
                                    if ComfStand_value in [13, 14] and ComfMod_value in [0, 1]:
                                        continue
                                    elif ComfStand_value not in [13, 14] and ComfMod_value in [0.1, 0.2, 0.3, 0.4, 0.5, 1.1, 1.2, 1.3, 1.4, 1.5]:
                                        continue
                                    elif ComfStand_value == 21 and ComfMod_value not in [2, 3]:
                                        continue
                                    else:
                                        SetInputData[0].Program_Line_3 = 'set ComfMod = ' + str(ComfMod_value)
                                        for HVACmode_value in self.HVACmode_List:
                                            SetInputData[0].Program_Line_4 = 'set HVACmode = ' + str(HVACmode_value)
                                            if HVACmode_value == 0:
                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                 self.ASTtol_value_to,
                                                                                 self.ASTtol_value_steps):
                                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                                    # SetComfTemp[0].Program_Line_2 = f'set ComfTemp = PMOT*{str(self.CustAST_m)}+{str(self.CustAST_n)}'
                                                    SetAppLimits[0].Program_Line_2 = f'set ACSTaul = {str(self.CustAST_ACSTaul)}'
                                                    SetAppLimits[0].Program_Line_3 = f'set ACSTall = {str(self.CustAST_ACSTall)}'
                                                    SetAppLimits[0].Program_Line_4 = f'set AHSTaul = {str(self.CustAST_AHSTaul)}'
                                                    SetAppLimits[0].Program_Line_5 = f'set AHSTall = {str(self.CustAST_AHSTall)}'
                                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)
                                                    ApplyCAT[0].Program_Line_4 = f'set ACSToffset = {str(self.CustAST_ACSToffset)} + {str(self.CATcoolOffset)}'
                                                    ApplyCAT[0].Program_Line_5 = f'set AHSToffset = {str(self.CustAST_AHSToffset)} + {str(self.CATheatOffset)}'
                                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                                    SetAST[0].Program_Line_2 = 'set m = ' + str(self.CustAST_m)
                                                    SetAST[0].Program_Line_3 = 'set n = ' + str(self.CustAST_n)
                                                    outputname = (
                                                            filename
                                                            + ComfStand_dict[ComfStand_value]
                                                            + '[CA_' + str(CAT_value)
                                                            + '[CM_' + str(ComfMod_value)
                                                            + '[HM_' + str(HVACmode_value)
                                                            + '[VC_X'
                                                            + '[VO_X'
                                                            + '[MT_X'
                                                            + '[MW_X'
                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                            + suffix
                                                            + '.idf'
                                                    )
                                                    if verboseMode:
                                                        print(outputname)
                                                        # time.sleep(0.1)
                                                        # pbar.update(1)
                                                    idf1.savecopy(outputname)
                                                    self.output_idf_dict.update({outputname: idf1})
                                            else:
                                                for VentCtrl_value in self.VentCtrl_List:
                                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + str(VentCtrl_value)
                                                    if HVACmode_value == 2:
                                                        if VentCtrl_value == 2 or VentCtrl_value == 3:
                                                            SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + str(self.MaxTempDiffVOF)
                                                            SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + str(self.MinTempDiffVOF)
                                                            SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + str(self.MultiplierVOF)
                                                    for VSToffset_value in self.VSToffset_List:
                                                        SetInputData[0].Program_Line_6 = 'set VSToffset = ' + str(
                                                            VSToffset_value)
                                                        for MinOToffset_value in self.MinOToffset_List:
                                                            SetInputData[0].Program_Line_7 = 'set MinOToffset = ' + str(
                                                                MinOToffset_value)
                                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                                SetInputData[0].Program_Line_8 = 'set MaxWindSpeed = ' + str(MaxWindSpeed_value)
                                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
                                                                                                 self.ASTtol_value_to,
                                                                                                 self.ASTtol_value_steps):
                                                                    SetInputData[0].Program_Line_9 = 'set ACSTtol = ' + str(-ASTtol_value)
                                                                    SetInputData[0].Program_Line_10 = 'set AHSTtol = ' + str(ASTtol_value)
                                                                    SetInputData[0].Program_Line_11 = 'set CoolSeasonStart = ' + str(self.CoolSeasonStart)
                                                                    SetInputData[0].Program_Line_12 = 'set CoolSeasonEnd = ' + str(self.CoolSeasonEnd)
                                                                    # SetComfTemp[0].Program_Line_2 = f'set ComfTemp = PMOT*{str(self.CustAST_m)}+{str(self.CustAST_n)}'
                                                                    SetAppLimits[0].Program_Line_2 = f'set ACSTaul = {str(self.CustAST_ACSTaul)}'
                                                                    SetAppLimits[0].Program_Line_3 = f'set ACSTall = {str(self.CustAST_ACSTall)}'
                                                                    SetAppLimits[0].Program_Line_4 = f'set AHSTaul = {str(self.CustAST_AHSTaul)}'
                                                                    SetAppLimits[0].Program_Line_5 = f'set AHSTall = {str(self.CustAST_AHSTall)}'

                                                                    ApplyCAT[0].Program_Line_1 = 'set CATcoolOffset = ' + str(self.CATcoolOffset)
                                                                    ApplyCAT[0].Program_Line_2 = 'set CATheatOffset = ' + str(self.CATheatOffset)
                                                                    ApplyCAT[0].Program_Line_4 = f'set ACSToffset = {str(self.CustAST_ACSToffset)} + {str(self.CATcoolOffset)}'
                                                                    ApplyCAT[0].Program_Line_5 = f'set AHSToffset = {str(self.CustAST_AHSToffset)} + {str(self.CATheatOffset)}'
                                                                    SetAST[0].Program_Line_1 = 'set SetpointAcc = ' + str(self.SetpointAcc)
                                                                    SetAST[0].Program_Line_2 = 'set m = ' + str(self.CustAST_m)
                                                                    SetAST[0].Program_Line_3 = 'set n = ' + str(self.CustAST_n)

                                                                    outputname = (
                                                                            filename
                                                                            + ComfStand_dict[ComfStand_value]
                                                                            + '[CA_' + str(CAT_value)
                                                                            + '[CM_' + str(ComfMod_value)
                                                                            + '[HM_' + str(HVACmode_value)
                                                                            + '[VC_' + str(VentCtrl_value)
                                                                            + '[VO_' + str(VSToffset_value)
                                                                            + '[MT_' + str(MinOToffset_value)
                                                                            + '[MW_' + str(MaxWindSpeed_value)
                                                                            + '[AT_' + str(round(ASTtol_value, 2))
                                                                            + suffix
                                                                            + '.idf'
                                                                    )
                                                                    if verboseMode:
                                                                        print(outputname)
                                                                        # time.sleep(0.1)
                                                                        # pbar.update(1)
                                                                    idf1.savecopy(outputname)
                                                                    self.output_idf_dict.update({outputname: idf1})
            elif TempCtrl.lower() == 'pmv':
                SetInputData[0].Program_Line_4 = 'set HVACmode = 0'
                outputname = (
                        filename
                        + '[CS_PMV'
                        + '[CA_X'
                        + '[CM_X'
                        + '[HM_0'
                        + '[VC_X'
                        + '[VO_X'
                        + '[MT_X'
                        + '[MW_X'
                        + '[AT_X'
                        + suffix
                        + '.idf'
                )
                if verboseMode:
                    print(outputname)
                    # time.sleep(0.1)
                    # pbar.update(1)
                idf1.savecopy(outputname)
                self.output_idf_dict.update({outputname: idf1})
        # pbar.close()
    elif confirmGen == False:
        if verboseMode:
            print('IDF generation has been shut down')

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    for file in filelist_pymod:
        os.remove(file)

    # del SetInputData



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import numpy as np


def descriptor_has_options(values):
    #Checking value entered is a list containing floats or a tuple containing the minimum and maximum values
    descriptor_has_options = False
    if type(values) == tuple and len(values) == 2 and all([type(i) == float or type(i) == int or type(i) == np.float64 for i in values]):
        pass
    elif type(values) == list and all(type(j) == float or type(j) == int or type(j) == np.float64 for j in values):
        descriptor_has_options = True
    else:
        raise ValueError('values argument must be, FOR ALL CASES, '
                         'a list containing int or float, '
                         'or a tuple which contains the minimum and maximum values for the range')
    return descriptor_has_options


import pandas as pd
import ast
from datetime import datetime, timedelta


def expand_to_hourly_dataframe(
        df: pd.DataFrame,
        parameter_columns: list,
        start_date: str = '2024-01-01 01',
        hourly_columns: list = None,
):
    """
    Expands a dataframe with hourly data columns into an hourly dataframe.

    Parameters:
    df (pd.DataFrame): The input dataframe containing parameters and hourly data columns.
    parameter_columns (list): The list of column names that contain input parameters.
    start_date (str): The start date and time in the format 'YYYY-MM-DD HH'.

    Returns:
    pd.DataFrame: The expanded dataframe with an additional datetime column.
    """

    # Identify columns with hourly data
    if hourly_columns is None:
        hourly_columns = identify_hourly_columns(df)

    # print(f"Hourly columns identified: {hourly_columns}")

    # Keep only parameter columns and hourly columns
    df_subset = df[parameter_columns + hourly_columns].copy()

    # Convert string representations of lists into actual lists
    for col in hourly_columns:
        # print(f"Processing column: {col}")
        if not df_subset[col].empty and isinstance(df_subset[col].iloc[0], str):
            print(f"First element of '{col}' is a string. Attempting evaluation...")
            evaluated_values = []
            for value in df_subset[col]:
                try:
                    evaluated_values.append(ast.literal_eval(value.strip()))
                except (ValueError, TypeError, SyntaxError, Exception) as e:
                    print(f"Error evaluating in column '{col}': '{value}' - {e}")
                    evaluated_values.append(None)
            df_subset[col] = evaluated_values
        else:
            continue
            # print(f"First element of '{col}' is not a string. Assuming it's already a list.")


    # Convert start_date to datetime object
    start_datetime = datetime.strptime(start_date, '%Y-%m-%d %H')

    # Function to expand the dataframe for hourly data
    def expand_hourly_data(row):
        num_hours = len(row[hourly_columns[0]])
        expanded_rows = {col: [row[col]] * num_hours for col in parameter_columns}
        expanded_rows['hour'] = list(range(1, num_hours + 1))
        expanded_rows['datetime'] = [start_datetime + timedelta(hours=i) for i in range(num_hours)]
        for col in hourly_columns:
            expanded_rows[col] = row[col]
        return pd.DataFrame(expanded_rows)

    # Apply the function to each row and concatenate the results
    expanded_df = pd.concat(df_subset.apply(expand_hourly_data, axis=1).to_list(), ignore_index=True)

    return expanded_df


def identify_hourly_columns(df):
    """
    Identifies the columns which contains strings representing lists.

    :param df: the pandas DataFrame
    :return: the list of column names
    """
    hourly_columns = [
        col for col in df.columns if
        df[col].apply(lambda x: isinstance(x, str) and x.startswith('[') and x.endswith(']')).all()
    ]

    if not hourly_columns:
        hourly_columns = [
            col for col in df.columns if
            df[col].astype(str).apply(lambda x: x.strip().startswith('[') and x.strip().endswith(']')).all()
        ]

    return hourly_columns

def make_all_combinations(parameters_values_dict: dict) -> pd.DataFrame:
    """
    Takes all values from all the parameters and return a pandas DataFrame with all possible combinations.

    :param parameters_values_dict: a dictionary in the format {'parameter name': list_of_values}
    :return: a pandas DataFrame with all possible combinations
    """
    from itertools import product
    combinations = list(product(*parameters_values_dict.values()))
    parameters_values_df = pd.DataFrame(combinations, columns=parameters_values_dict.keys())
    return parameters_values_df



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

def morph_epws(
        fwg_path: str,
        epw_filepaths: any = 'all',
        delete_morphing_files: bool = True,
        verbose: bool = False,
        fwg_options_gcms: str = None,
        fwg_options_ensemble: int = 1,
        fwg_options_month_transition_hours: int = 72,
        fwg_options_do_multithread_computation: bool = True,
        fwg_options_interpolation_method_id: int = 0,
        fwg_options_do_limit_variables: bool = True,
):
    """
    This function is a wrapper for the use of the FutureWeatherGenerator tool (https://future-weather-generator.adai.pt/),
    developed by Eugènio Rodrigues et al., in command line.
    It will move the epws in the working directory to a folder with the same name,
    will morph them within their respective directories,
    then will rename them using the original epw filename with the suffix depending on the scenario
    (e.j. the future scenarios for Granada.epw will be Granada_ssp126_2050.epw, Granada_ssp126_2080.epw, etc.)
    and move them back to the parent directory.
    If requested, the files generated from the moprhing process will be deleted.

    :param fwg_path: The path to the FutureWeatherGenerator tool.
    :param epw_filepaths: A list of epw filenames to be morphed.
    :param delete_morphing_files: True to delete all files resulting from the morphing process
        except .epw files.
    :param verbose: True to print on screen the performed actions.
    :param fwg_options_gcms: The GCMs separated by a comma (without white space).
    :param fwg_options_ensemble: Option to create an ensemble from the selected models: 0 is no, 1 is yes.
    :param fwg_options_month_transition_hours: the number of hours to smooth the transition between months: 0 to 336 hours.
    :param fwg_options_do_multithread_computation: True to do the computations in multithread.
    :param fwg_options_interpolation_method_id: Selects the grid method: 0 – bilinear interpolation, 1 – an average of the four nearest points; 2 – nearest point.
    :param fwg_options_do_limit_variables: True to bound each generated variable to its physical limits
    """
    import os
    import shutil
    import subprocess

    # Get the absolute path of the current working directory
    cwd = os.getcwd()

    # Get a list of all .epw files in the current working directory
    if epw_filepaths == 'all':
        epw_files = [file for file in os.listdir() if file.endswith(".epw")]
    else:
        epw_files = epw_filepaths

    if fwg_options_gcms is None:
        fwg_options_gcms = 'BCC_CSM2_MR,CanESM5,CanESM5_1,CanESM5_CanOE,CAS_ESM2_0,CMCC_ESM2,CNRM_CM6_1_HR,CNRM_ESM2_1,EC_Earth3,EC_Earth3_Veg,FGOALS_g3,GISS_E2_1_G,GISS_E2_1_H,GISS_E2_2_G,IPSL_CM6A_LR,MIROC_ES2H,MIROC_ES2L,MIROC6,MRI_ESM2_0,UKESM1_0_LL'

    # List to store tuples of directories, EPW file names, absolute paths of folders, and absolute paths of EPW files
    directory_file_paths = []

    # Iterate through each .epw file
    for epw_file in epw_files:
        # Create a folder with the same name as the .epw file (without extension)
        folder_name = os.path.splitext(epw_file)[0]
        folder_path = os.path.join(cwd, folder_name)
        os.makedirs(folder_path, exist_ok=True)

        # Move the .epw file into the created folder
        epw_file_path = os.path.join(folder_path, epw_file)
        shutil.move(epw_file, epw_file_path)

        # Save the directory name, EPW file name, folder absolute path, and EPW file absolute path in the list
        directory_file_paths.append((folder_name, epw_file, folder_path, epw_file_path))
        if verbose:
            print(f"Moved {epw_file} to folder {folder_name}")

    for folder_name, epw_name, folder_path, epw_path in [i for i in directory_file_paths]:
        # directory = os.path.dirname(epw)
        # directory = r'D:\OneDrive - ZER0CEM\Python\testing_morphing_python'
        subprocess.run(f'java -cp '
                       f'"{fwg_path}" '
                       f'futureweathergenerator.Morph "{epw_path}" '
                       f'{fwg_options_gcms} '
                       f'{fwg_options_ensemble} '
                       f'{fwg_options_month_transition_hours} '
                       f'"{folder_path}/" '
                       f'{fwg_options_do_multithread_computation} '
                       f'{fwg_options_interpolation_method_id} '
                       f'{fwg_options_do_limit_variables}')


    # Search for new EPW files in the folder paths and rename them according to the pattern
    scenario_strings = ['ssp126_2050', 'ssp126_2080', 'ssp245_2050', 'ssp245_2080', 'ssp370_2050', 'ssp370_2080',
                        'ssp585_2050', 'ssp585_2080']

    for folder_name, epw_name, folder_path, epw_path in [i for i in directory_file_paths]:
        # List files in the folder path
        files_in_folder = [ i for i in os.listdir(folder_path) if i.endswith('.epw')]

        for file_name in files_in_folder:
            # Check if the file name contains any of the scenario strings
            for scenario in scenario_strings:
                if scenario in file_name:
                    # Rename the file following the specified pattern
                    new_file_name = f"{folder_name}_{scenario}.epw"
                    old_file_path = os.path.join(folder_path, file_name)
                    new_file_path = os.path.join(folder_path, new_file_name)
                    os.rename(old_file_path, new_file_path)
                    if verbose:
                        print(f"Renamed {file_name} to {new_file_name}")

                    # Move the renamed file to the parent folder
                    parent_folder_path = os.path.dirname(folder_path)
                    moved_file_path = os.path.join(parent_folder_path, new_file_name)
                    shutil.move(new_file_path, moved_file_path)
                    if verbose:
                        print(f"Moved {new_file_name} to {parent_folder_path}")

        # Move the original EPW file to the parent folder
        original_file_path = os.path.join(folder_path, folder_name + ".epw")
        shutil.move(original_file_path, parent_folder_path)
        if verbose:
            print(f"Moved {folder_name}.epw to {parent_folder_path}")

    if delete_morphing_files:
        # Delete the folders created during the process
        for folder_name, epw_name, folder_path, epw_path in [i for i in directory_file_paths]:
            shutil.rmtree(folder_path)
            if verbose:
                print(f"Deleted folder: {folder_path}")




import pandas as pd
from besos import eppy_funcs as ef
from besos.evaluator import EvaluatorEP
from besos.parameters import Parameter, GenericSelector, CategoryParameter
from besos.problem import EPProblem
from besos.objectives import MeterReader

import numpy as np

import accim.sim.apmv_setpoints as apmv
import accim.parametric_and_optimisation.funcs_for_besos.param_apmv as bf



building = ef.get_building(
    'aPMV_testing_v01_no_script.idf'
)


[i for i in building.idfobjects['EnergyManagementSystem:Program']]

apmv.apply_apmv_setpoints(
    building=building,
    # setpoint_tolerance=0.15
)

apmv.set_zones_always_occupied(building)



##

# gv = [i for i in building.idfobjects['EnergyManagementSystem:GlobalVariable']]

building.newidfobject(
    key='OUTPUT:METER',
    Key_Name='Electricity:Facility',
    Reporting_Frequency='RunPeriod'
)

building.newidfobject(
    key='OUTPUT:METER',
    Key_Name='Electricity:HVAC',
    Reporting_Frequency='RunPeriod'
)


# available_outputs = print_available_outputs_mod(building)
##

[i for i in building.idfobjects['energymanagementsystem:program'] if 'set_zone_input_data' in i.Name]
# [i for i in building.idfobjects['energymanagementsystem:program']]

# def change_adaptive_coeff(building, value):
#     program = [i for i in building.idfobjects['ENERGYMANAGEMENTSYSTEM:PROGRAM'] if 'set_zone_input_data_Block1_Zone1' in i.Name][0]
#     program.Program_Line_1 = f'set adap_coeff_cooling_Block1_Zone1 = {value}'
#     program.Program_Line_2 = f'set adap_coeff_heating_Block1_Zone1 = {value}'
#
# def change_pmv_sp(building, value):
#     program = [i for i in building.idfobjects['ENERGYMANAGEMENTSYSTEM:PROGRAM'] if 'set_zone_input_data_Block1_Zone1' in i.Name][0]
#     program.Program_Line_3 = f'set pmv_cooling_sp_Block1_Zone1 = {value}'
#     program.Program_Line_4 = f'set pmv_heating_sp_Block1_Zone1 = {value}'
#
# def change_adaptive_coeff_all_zones(building, value):
#     programs = [i for i in building.idfobjects['ENERGYMANAGEMENTSYSTEM:PROGRAM'] if 'set_zone_input_data' in i.Name]
#     people = [people.Zone_or_ZoneList_Name.replace(':', '_') for people in building.idfobjects['People']]
#     for program in programs:
#         for ppl in people:
#             if ppl in program.Name:
#                 program.Program_Line_1 = f'set adap_coeff_cooling_{ppl} = {value}'
#                 program.Program_Line_2 = f'set adap_coeff_heating_{ppl} = {value}'

    # program.Program_Line_1 = f'set adap_coeff_cooling_Block1_Zone1 = {value}'
    # program.Program_Line_2 = f'set adap_coeff_heating_Block1_Zone1 = {value}'

##
# programs = [i for i in building.idfobjects['ENERGYMANAGEMENTSYSTEM:PROGRAM'] if 'set_zone_input_data' in i.Name]
# people = [people.Zone_or_ZoneList_Name.replace(':', '_') for people in building.idfobjects['People']]
# value = 0.66
# for program in programs:
#     for ppl in people:
#         if ppl in program.Name:
#             program.Program_Line_1 = f'set adap_coeff_cooling_{ppl} = {value}'
#             program.Program_Line_2 = f'set adap_coeff_heating_{ppl} = {value}'
#             # program.Program_Line_1 = "set adap_coeff_cooling_% = %" % (ppl, value)
#             # program.Program_Line_2 = "set adap_coeff_heating_% = %" % (ppl, value)
#             # program.Program_Line_1 = 'set adap_coeff_cooling_'+ppl+' = '+str(value),
#             # program.Program_Line_2 = 'set adap_coeff_heating_'+ppl+' = '+str(value),

# program = [i for i in building.idfobjects['ENERGYMANAGEMENTSYSTEM:PROGRAM'] if 'set_zone_input_data_Block1_Zone1' in i.Name][0]
# value = -0.5
# program.Program_Line_1 = f'set adap_coeff_cooling_Block1_Zone1 = {value}'
# program.Program_Line_2 = f'set adap_coeff_heating_Block1_Zone1 = {value}'

##

# [i for i in building.idfobjects['output:variable'] if 'Facility' in i.Variable_Name]
#
#
# totalhvacoutput = []
# for i in range(len(available_outputs.variablereaderlist)):
#     if 'Facility' in available_outputs.variablereaderlist[i][1]:
#         totalhvacoutput.append(available_outputs.variablereaderlist[i])
# totalhvacoutput = totalhvacoutput[0]

##


# def change_adaptive_coeff(building, value):
#     # program = [p for p in building.idfobjects['EnergyManagementSystem:Program'] if 'apply_aPMV' in p.Name][0]
#     # program.Program_Line_1 = f'set adap_coeff = {value}'
#     apmv.change_adaptive_coeff(building=building, value=value)
#     return
#
# def change_PMV_setpoints(building, value):
#     # program = [p for p in building.idfobjects['EnergyManagementSystem:Program'] if 'apply_aPMV' in p.Name][0]
#     # program.Program_Line_2 = f'set PMV_H_SP = {-value}'
#     # program.Program_Line_3 = f'set PMV_C_SP = {value}'
#     apmv.change_PMV_setpoints(building=building, value=value)
#     return


##

adaptive_coeff_range = [round(i, 2) for i in np.arange(-1.0, 0, 0.1)]
adaptive_coeff_range.extend([round(i, 2) for i in np.arange(0, 1.1, 0.1)])
len(adaptive_coeff_range)

pmv_range = [round(i, 2) for i in np.arange(0.3, 0.90, 0.05)]
len(pmv_range)
pmv_full = [[i]*len(adaptive_coeff_range) for i in pmv_range]
pmv_full = np.array(pmv_full).flatten().tolist()

samples = pd.DataFrame(
    {
        "Adaptive coefficient": adaptive_coeff_range*len(pmv_range),
        "PMV": pmv_full,
    }
)

samples_short = samples[:5]

parameters = [
    Parameter(
        name='Adaptive coefficient',
        # selector=GenericSelector(set=change_adaptive_coeff),
        selector=GenericSelector(set=bf.change_adaptive_coeff_all_zones),
        value_descriptors=CategoryParameter(options=adaptive_coeff_range)
    ),
    Parameter(
        name='PMV',
        # selector=GenericSelector(set=change_PMV_setpoints),
        selector=GenericSelector(set=bf.change_pmv_setpoint_all_zones),
        value_descriptors=CategoryParameter(options=pmv_range)
    ),
]

## Objectives

# objs_comfhours = []
# for i in range(len(available_outputs.variablereaderlist)):
#     if 'hour' in available_outputs.variablereaderlist[i][1].lower():
#         objs_comfhours.append(
#             VariableReader(
#                 key_value=available_outputs.variablereaderlist[i][0],
#                 variable_name=available_outputs.variablereaderlist[i][1],
#                 frequency=available_outputs.variablereaderlist[i][2],
#                 name=available_outputs.variablereaderlist[i][1]
#             )
#         )

##

objectives = [
    MeterReader("Electricity:Facility", name="Total Electricity Usage"),
    MeterReader("Electricity:HVAC", name="HVAC Electricity Usage"),

    # VariableReader(
    #     key_value='Whole Building',
    #     variable_name='Facility Total HVAC Electricity Demand Rate',
    #     frequency='Hourly',
    #     name='HVAC Electricity usage'
    # )
]

problem = EPProblem(
    inputs=parameters,
    # outputs=objectives+objs_comfhours
    outputs=objectives
)


# inputs = sampling.dist_sampler(sampling.full_factorial, problem, num_samples=len(adaptive_coeff_range), level=len(pmv_range))
# inputs

# samples = pd.DataFrame(
#     {
#         "Thickness": [x / 10 for x in range(1, 10)] * 2,
#         "Watts": [8, 10, 12] * 6,
#         "wwr": [0.25, 0.5] * 9,
#     }
# )

# bundle all of the different selectors into a single list of parameters

# parameters = [
#     Parameter(selector=x) for x in (insulation_idf, lights_selector, window_to_wall)
# ]

evaluator = EvaluatorEP(
    problem=problem,
    building=building,
    out_dir='outdir'
)

outputs = evaluator.df_apply(
    samples_short,
    keep_input=True,
    keep_dirs=True,
    # out_dir='outdir',
    processes=5
)

# outputs_mod = outputs
# outputs_mod['energy ratio'] = outputs_mod['HVAC Electricity Usage'] / outputs_mod['Total Electricity Usage']



# generated_buildings = [evaluator.generate_building(df=samples_short, index=i, file_name=f'short_sample_row_{i}') for i in range(5)]
evaluator.generate_building(df=samples_short, index=0, file_name='num_0')
evaluator.generate_building(df=samples_short, index=1, file_name='num_1')
evaluator.generate_building(df=samples_short, index=2, file_name='num_2')
evaluator.generate_building(df=samples_short, index=3, file_name='num_3')
evaluator.generate_building(df=samples_short, index=4, file_name='num_4')


# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Class for accim."""

class accimJob():
    """Class to start the process to add the class ``accim.sim.accis.addAccis``.

    :param idf_class_instance: the filename of the idf
    :param ScriptType: Inherited from class ``accim.sim.accis.addAccis``
    :param EnergyPlus_version: Inherited from class ``accim.sim.accis.addAccis``
    :param TempCtrl: Inherited from class ``accim.sim.accis.addAccis``
    :param verboseMode: Inherited from class ``accim.sim.accis.addAccis``
    :param accimNotWorking: True if problems detected in class ``accim.sim.accis.addAccis``
    """
    from os import listdir
    import numpy

    # from accim.sim.accim_IDFgeneration import \
    #     inputData,\
    #     genIDF
    from accim.sim.accim_Base import \
        setComfFieldsPeople, \
        saveaccim, \
        setPMVsetpoint, \
        addControlFilesObjects, \
        addOutputVariableDictionaryObject, \
        addOutputEnergyManagementSystem
    from accim.sim.accim_Base_EMS import \
        addEMSActuatorsBase, \
        addEMSOutputVariableBase, \
        addEMSPCMBase, \
        addEMSProgramsBase, \
        addEMSSensorsBase, \
        addGlobVarList, \
        addIntVarList, \
        addOutputVariablesStandard, \
        addOutputVariablesSimplified, \
        addOutputVariablesDetailed, \
        removeExistingOutputVariables, \
        removeDuplicatedOutputVariables, \
        outputsSpecified, \
        genOutputDataframe, \
        takeOutputDataFrame, \
        makeAverages

    from accim.sim.accim_ExistingHVAC import \
        addForscriptSchExistHVAC
    from accim.sim.accim_ExistingHVAC_EMS import \
        addEMSSensorsExisHVAC
    from accim.sim.accim_VRFsystem import \
        addBaseSchedules, \
        addCurveObj, \
        addDetHVACobj, \
        addForscriptSchVRFsystem, \
        addOpTempTherm, \
        addVRFsystemSch, \
        checkVentIsOn, \
        setAvailSchOn
    from accim.sim.accim_VRFsystem_EMS import \
        addEMSSensorsVRFsystem
    from accim.sim.utils import scan_zones

    def __init__(self,
                 idf_class_instance,
                 ScriptType: str = None,
                 EnergyPlus_version: str = None,
                 TempCtrl: str = None,
                 verboseMode: bool = True,
                 ):
        """
        Constructor method.
        """
        import eppy
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        # IDF.setiddname(api_environment.EnergyPlusInputIddPath)
        # idf1 = IDF(api_environment.EnergyPlusInputIdfPath)

        self.idf1 = idf_class_instance

        # Scanning occupied zones using func
        self.scan_zones()
        # # Scanning occupied zones
        # self.occupiedZones_orig = []
        #
        # # Check if model comes from OpenStudio
        #
        # # Check if ZoneList or SpaceList are used
        # occupiedZones_orig_osm = []
        #
        # self.spacelist_use = False
        # try:
        #     if len(self.idf1.idfobjects['SPACELIST']) > 0:
        #         self.spacelist_use = True
        #         self.spacenames_for_ems_uniquekey = []
        #         self.spacenames_for_ems_name = []
        #         self.spacenames_for_ems_uniquekey_people = []
        #         self.zonenames_for_ems_with_sl = []
        #         for people in self.idf1.idfobjects['PEOPLE']:
        #             for spacelist in [i for i in self.idf1.idfobjects['SPACELIST'] if i.Name == people.Zone_or_ZoneList_or_Space_or_SpaceList_Name]:
        #                 for space in [i for i in self.idf1.idfobjects['SPACE'] if i.Space_Type == spacelist.Name]:
        #                     self.spacenames_for_ems_uniquekey.append(f'{space.Name} {spacelist.Name}')
        #                     self.spacenames_for_ems_name.append(space.Name)
        #                     self.spacenames_for_ems_uniquekey_people.append(f'{space.Name} {people.Name}')
        #                     occupiedZones_orig_osm.append(space.Zone_Name)
        #                     for zone in [i for i in self.idf1.idfobjects['ZONE'] if space.Zone_Name == i.Name]:
        #                         self.zonenames_for_ems_with_sl.append(zone.Name)
        # except KeyError:
        #     idd = '-'.join([str(i) for i in self.idf1.idd_version])
        #     print('Searching Spacelist objects returned KeyError. '
        #           f'That means these are not supported in the EnergyPlus version {idd}')
        #
        # # occupiedZones_orig_osm = []
        # # if len([h for h in self.idf1.idfobjects['zonelist']]) > 0:
        # #     if len(self.idf1.idfobjects['zone']) == 1:
        # #         no_of_zones = range(1, 2)
        # #     else:
        # #         no_of_zones = range(1, len(self.idf1.idfobjects['zone']))
        # #
        # #     for i in no_of_zones:
        # #         for j in self.idf1.idfobjects['zonelist']:
        # #             for k in self.idf1.idfobjects['zone']:
        # #                 if k.Name in j[f'Zone_{i}_Name']:
        # #                     occupiedZones_orig_osm.append(k.Name)
        # #
        # #     # for i in self.idf1.idfobjects['zone']:
        # #     #     if all(i.Name not in [j for j in occupiedZones_orig_osm]):
        # #     #         occupiedZones_orig_osm.append(i.Name)
        #
        # occupiedZones_orig_dsb = []
        # for i in self.idf1.idfobjects['ZONE']:
        #     for k in self.idf1.idfobjects['PEOPLE']:
        #         try:
        #             if i.Name == k.Zone_or_ZoneList_or_Space_or_SpaceList_Name:
        #                 occupiedZones_orig_dsb.append(i.Name.upper())
        #         except eppy.bunch_subclass.BadEPFieldError:
        #             if i.Name == k.Zone_or_ZoneList_Name:
        #                 occupiedZones_orig_dsb.append(i.Name.upper())
        #
        # if self.spacelist_use:
        #     self.occupiedZones_orig = occupiedZones_orig_osm
        #     self.occupiedZones = [i.replace(' ', '_') for i in self.occupiedZones_orig]
        #     self.origin_dsb = False
        #     self.ems_objs_name = self.spacenames_for_ems_name
        #     self.ems_objs_key = self.spacenames_for_ems_uniquekey
        #     self.ems_zonenames = self.zonenames_for_ems_with_sl
        #     self.ems_zonenames_underscore = [i.replace(' ', '_') for i in self.ems_zonenames]
        # else:
        #     self.occupiedZones_orig = occupiedZones_orig_dsb
        #     self.occupiedZones = [i.replace(':', '_') for i in self.occupiedZones_orig]
        #     self.origin_dsb = True
        #     self.ems_objs_name = self.occupiedZones
        #     self.ems_objs_key = self.occupiedZones_orig
        #     self.ems_zonenames = self.occupiedZones_orig
        #     self.ems_zonenames_underscore = self.occupiedZones_orig


        if verboseMode:
            print(f'The occupied zones in the model {idf_class_instance} are:')
            print(*self.occupiedZones_orig, sep="\n")

        self.ismixedmode = False

        if (ScriptType.lower() == 'vrfsystem_mm' or
            ScriptType.lower() == 'vrf_mm' or
            ScriptType.lower() == 'existinghvac_mm' or
            ScriptType.lower() == 'ex_mm'
        ):
            self.ismixedmode = True
            self.windownamelist_orig = []

            for i in [window.Name for window in
                      self.idf1.idfobjects
                      ['AirflowNetwork:MultiZone:Component:DetailedOpening']
                      if window.Name.endswith('_Win')
                      or window.Name.endswith('_Door')
                      ]:
                for k in self.occupiedZones_orig:
                    if i.split('_')[0].lower() in k.lower():
                        self.windownamelist_orig.append(i)

            self.windownamelist = [i.replace(':', '_') for i in self.windownamelist_orig]

            # print(self.windownamelist_orig)
            self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
            # print(self.windownamelist_orig_split)
            if verboseMode:
                print(f'The windows and doors in the model {idf_class_instance} are:')
                print(*self.windownamelist, sep="\n")

        if 'vrf' in ScriptType.lower():
            self.zonenames = self.occupiedZones
            self.zonenames_orig = self.occupiedZones_orig
            if verboseMode:
                print(f'The zones in the model {idf_class_instance} are:')
                print(*self.zonenames, sep="\n")

        elif 'ex' in ScriptType.lower():
            TSPtypes = [
                'ThermostatSetpoint:SingleHeating',
                'ThermostatSetpoint:SingleCooling',
                # ThermostatSetpoint:SingleHeatingOrCooling objects are not supported
                # 'ThermostatSetpoint:SingleHeatingOrCooling',
                'ThermostatSetpoint:DualSetpoint'
            ]
            self.ZCTlist = [i for i in self.idf1.idfobjects['ZONECONTROL:THERMOSTAT']]

            self.HVACzonelist = []

            for i in range(len(TSPtypes)):
                temp1 = []
                temp2 = []
                temp3 = []
                if len(self.idf1.idfobjects[TSPtypes[i]]) > 0:
                    for j in range(len(self.ZCTlist)):
                        if self.ZCTlist[j].Control_1_Object_Type in TSPtypes[i]:
                            temp1.append(self.ZCTlist[j].Zone_or_ZoneList_Name.upper())
                            if ':' in self.ZCTlist[j].Zone_or_ZoneList_Name:
                                temp2.append(self.ZCTlist[j].Zone_or_ZoneList_Name.upper().replace(":", "_"))
                            else:
                                temp2.append([i.Name.upper() for i in self.idf1.idfobjects['SPACE'] if i.Zone_Name.upper() == self.ZCTlist[j].Zone_or_ZoneList_Name.upper()][0])
                            temp3.append(self.ZCTlist[j].Control_1_Name)
                self.HVACzonelist.append([TSPtypes[i], temp1, temp2, temp3])
            del temp1, temp2, temp3

            if verboseMode:
                for i in range(len(self.HVACzonelist)):
                    if len(self.HVACzonelist[i][3]) == 0:
                        print(f'There are no {self.HVACzonelist[i][0]} objects in the model')
                    else:
                        print(f'Regarding {self.HVACzonelist[i][0]} objects:')
                        print(f'The zones with {self.HVACzonelist[i][0]} are:')
                        print(*self.HVACzonelist[i][1], sep="\n")
                        print(f'And the existing ThermostatSetpoint objects related to {self.HVACzonelist[i][0]} are:')
                        print(*self.HVACzonelist[i][3], sep="\n")

            self.zonenames_orig = []
            # todo currently all zones regardless the single or dual thermostat object are merged in zonenames_orig;
            #  this would be desirable to be amended

            for i in range(len(self.HVACzonelist)):
                for k in range(len(self.HVACzonelist[i][1])):
                    if self.HVACzonelist[i][1][k] in self.zonenames_orig:
                        continue
                    else:
                        self.zonenames_orig.append(self.HVACzonelist[i][1][k])
            if self.origin_dsb:
                self.zonenames = [i.replace(':', '_') for i in self.zonenames_orig]
            else:
                self.zonenames = [i.replace(' ', '_') for i in self.zonenames_orig]

            if ScriptType.lower() == 'existinghvac_mm' or ScriptType.lower() == 'ex_mm':

                self.HVACdict = {
                    # todo if there is a Coil:Heating:Whatever and another Coil:Heating:DifferentWhatever
                    #  coils and windows sensors will be duplicated and simulation will crash; it needs to be solved.

                    # Group Heating and Cooling Coils
                    'Coil:Cooling:Water': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:Water:DetailedGeometry': 'Cooling Coil Total Cooling Rate',
                    # not supported
                    # 'CoilSystem:Cooling:Water:HeatExchangerAssisted':'',
                    'CoilSystem:Cooling:Water': 'Coil System Water Total Cooling Rate',
                    'Coil:Heating:Water': 'Heating Coil Heating Energy',
                    'Coil:Heating:Steam': 'Heating Coil Heating Energy',
                    'Coil:Heating:Electric': 'Heating Coil Heating Energy',
                    'Coil:Heating:Electric:MultiStage': 'Heating Coil Heating Energy',
                    'Coil:Heating:Desuperheater': 'Heating Coil Heating Energy',
                    'Coil:Cooling:DX:VariableRefrigerantFlow': 'Cooling Coil Total Cooling Rate',
                    'Coil:Heating:DX:VariableRefrigerantFlow': 'Heating Coil Heating Energy',
                    'Coil:Cooling:DX:VariableRefrigerantFlow:FluidTemperatureControl': 'Cooling Coil Total Cooling Rate',
                    'Coil:Heating:DX:VariableRefrigerantFlow:FluidTemperatureControl': 'Heating Coil Heating Energy',
                    'Coil:Heating:Fuel': 'Heating Coil Heating Energy',
                    'Coil:Heating:Gas:MultiStage': 'Heating Coil Heating Energy',
                    'Coil:Cooling:DX:SingleSpeed': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:TwoSpeed': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:TwoStageWithHumidityControlMode': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:MultiSpeed': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:VariableSpeed': 'Cooling Coil Total Cooling Rate',
                    # not supported
                    # 'CoilPerformance:DX:Cooling': '',
                    'Coil:Heating:DX:SingleSpeed': 'Heating Coil Heating Energy',
                    'Coil:Heating:DX:MultiSpeed': 'Heating Coil Heating Energy',
                    'Coil:Heating:DX:VariableSpeed': 'Heating Coil Heating Energy',
                    'Coil:WaterHeating:Desuperheater': 'Water Heater Heating Energy',
                    # not supported
                    # 'CoilSystem:Cooling:DX': '',
                    # 'CoilSystem:Heating:DX': '',
                    # 'CoilSystem:Cooling:DX:HeatExchangerAssisted': '',
                    # 'CoilSystem:IntegratedHeatPump:AirSource': '',
                    # 'Coil:WaterHeating:AirToWaterHeatPump:Pumped': 'Heating Coil Heating Energy',
                    # 'Coil:WaterHeating:AirToWaterHeatPump:Wrapped': 'Heating Coil Heating Energy',
                    # 'Coil:WaterHeating:AirToWaterHeatPump:VariableSpeed': 'Cooling Coil Electricity Energy',
                    'Coil:Cooling:WaterToAirHeatPump:ParameterEstimation': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:WaterToAirHeatPump:EquationFit': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit': 'Cooling Coil Total Cooling Rate',
                    'Coil:Heating:WaterToAirHeatPump:ParameterEstimation': 'Heating Coil Heating Energy',
                    'Coil:Heating:WaterToAirHeatPump:EquationFit': 'Heating Coil Heating Energy',
                    'Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit': 'Heating Coil Heating Energy',
                    'Coil:Cooling:DX:SingleSpeed:ThermalStorage': 'Cooling Coil Total Cooling Rate',
                    # not supported
                    # 'Secondary Coils of DX System and Heat Pump':'',
                    'Coil:Cooling:DX': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:CurveFit:Performance': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:CurveFit:OperatingMode': 'Cooling Coil Total Cooling Rate',
                    'Coil:Cooling:DX:CurveFit:Speed': 'Cooling Coil Total Cooling Rate',

                    # Group – Radiative / Convective Units
                    # todo many objects have heating or cooling outputs for the same field
                    'ZoneHVAC:Baseboard:RadiantConvective:Water': 'Baseboard Total Heating Rate',
                    'ZoneHVAC:Baseboard:RadiantConvective:Steam': 'Baseboard Total Heating Rate',
                    'ZoneHVAC:Baseboard:RadiantConvective:Electric': 'Baseboard Total Heating Rate',
                    'ZoneHVAC:CoolingPanel:RadiantConvective:Water': 'Cooling Panel Total Cooling Rate',
                    'ZoneHVAC:Baseboard:Convective:Water': 'Baseboard Total Heating Rate',
                    'ZoneHVAC:Baseboard:Convective:Electric': 'Baseboard Total Heating Rate',
                    # not supported
                    # ZoneHVAC:LowTemperatureRadiant:VariableFlow can be a chilled ceiling: Zone Radiant HVAC Cooling Rate
                    # also it can be a heated floor: Zone Radiant HVAC Heating Rate
                    # 'ZoneHVAC:LowTemperatureRadiant:VariableFlow': 'Zone Radiant HVAC Heating Energy',
                    # 'ZoneHVAC:LowTemperatureRadiant:ConstantFlow': 'Zone Radiant HVAC Heating Energy',
                    # 'ZoneHVAC:LowTemperatureRadiant:Electric': 'Zone Radiant HVAC Heating Energy',
                    # 'ZoneHVAC:LowTemperatureRadiant:SurfaceGroup': 'Zone Radiant HVAC Heating Energy',
                    # 'ZoneHVAC:HighTemperatureRadiant': 'Zone Radiant HVAC Heating Energy',
                    # 'ZoneHVAC:VentilatedSlab': '',
                    # 'ZoneHVAC:VentilatedSlab:SlabGroup': '',

                    # Group – Zone HVAC Air Loop Terminal Units
                    # 'AirTerminal:SingleDuct:ConstantVolume:Reheat': '',
                    # 'AirTerminal:SingleDuct:ConstantVolume:NoReheat': '',
                    # 'AirTerminal:SingleDuct:VAV:Reheat': '',
                    # 'AirTerminal:SingleDuct:VAV:Reheat:VariableSpeedFan': '',
                    # 'AirTerminal:SingleDuct:VAV:HeatAndCool:Reheat': '',
                    # 'AirTerminal:SingleDuct:VAV:NoReheat': '',
                    # 'AirTerminal:SingleDuct:VAV:HeatAndCool:NoReheat': '',
                    # 'AirTerminal:SingleDuct:SeriesPIU:Reheat': '',
                    # 'AirTerminal:SingleDuct:ParallelPIU:Reheat': '',
                    # 'AirTerminal:SingleDuct:ConstantVolume:FourPipeInduction': '',
                    # 'AirTerminal:SingleDuct:ConstantVolume:FourPipeBeam': '',
                    'AirTerminal:SingleDuct:ConstantVolume:CooledBeam': 'Zone Air Terminal Beam Chilled Water Energy',
                    # 'AirTerminal:SingleDuct:Mixer': '',
                    # 'AirTerminal:DualDuct:ConstantVolume': '',
                    # 'AirTerminal:DualDuct:VAV': '',
                    # 'AirTerminal:DualDuct:VAV:OutdoorAir': ''
                }

                HVACkeylist = list(self.HVACdict.keys())

                self.ExisHVAC = []

                for i in range(len(HVACkeylist)):
                    try:
                        temp = [i.Name for i in self.idf1.idfobjects[HVACkeylist[i]]]
                        temp_zone_orig = [i.Name.split(' ')[0] for i in self.idf1.idfobjects[HVACkeylist[i]]]
                        temp_zone = [i.Name.split(' ')[0].replace(':', '_') for i in self.idf1.idfobjects[HVACkeylist[i]]]
                        temp_win = []
                        for j in temp_zone:
                            for k in self.windownamelist:
                                if j in k:
                                    temp_win.append(k)
                        # hasta aqui
                        if len(temp) == 0:
                            continue
                        else:
                            self.ExisHVAC.append([HVACkeylist[i], temp, temp_zone_orig, temp_zone, temp_win])
                    except KeyError:
                        if verboseMode:
                            print(f'{HVACkeylist[i]} HVAC SYSTEM IS NOT SUPPORTED')
                        continue

                for i in range(len(self.ExisHVAC)):
                    for j in range(len(self.ExisHVAC[i][2])):
                        if self.ExisHVAC[i][2][j] not in self.zonenames_orig:
                            if verboseMode:
                                print(f'"{self.ExisHVAC[i][2][j]}" is not a valid room. \n'
                                      f'That means "{self.ExisHVAC[i][1][j]}" is not named following [HVAC Zone] [HVAC Element] pattern or HVAC element is shared')
                            # raise ValueError
                            self.accimNotWorking = True
                
                if verboseMode:
                    for i in range(len(self.ExisHVAC)):
                        print(f'The names of the existing {self.ExisHVAC[i][0]} objects are:')
                        print(*self.ExisHVAC[i][1], sep="\n")
                        print(f'The zones related to these {self.ExisHVAC[i][0]} objects are')
                        print(*self.ExisHVAC[i][2], sep='\n')
                        print(f'And the windows related to these {self.ExisHVAC[i][0]} objects are:')
                        print(*self.ExisHVAC[i][4], sep='\n')



# accim - Adaptive-Comfort-Control-Implemented Model
# Copyright (C) 2021-2025 Daniel Sánchez-García

# accim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# accim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Dictionaries to be used in multiple files"""



EMSOutputVariableZone_dict = {
    'Comfortable Hours_No Applicability': ['ComfHoursNoApp', 'H', 'Summed'],
    'Comfortable Hours_Applicability': ['ComfHours', 'H', 'Summed'],
    'Occupied Comfortable Hours_No Applicability': ['OccComfHoursNoApp', 'H', 'Summed'],
    'Occupied Discomfortable Hours_No Applicability': ['OccDiscomfHoursNoApp', 'H', 'Summed'],
    'Occupied Hours': ['OccHours', 'H', 'Summed'],
    'Discomfortable Applicable Hot Hours': ['DiscomfAppHotHours', 'H', 'Summed'],
    'Discomfortable Applicable Cold Hours': ['DiscomfAppColdHours', 'H', 'Summed'],
    'Discomfortable Non Applicable Hot Hours': ['DiscomfNonAppHotHours', 'H', 'Summed'],
    'Discomfortable Non Applicable Cold Hours': ['DiscomfNonAppColdHours', 'H', 'Summed'],
    'Zone Floor Area': ['ZoneFloorArea', 'm2', 'Averaged'],
    'Zone Air Volume': ['ZoneAirVolume', 'm3', 'Averaged'],
    'People Occupant Count': ['Occ_count', '', 'Summed'],
}




from eppy.modeleditor import IDF
from accim.sim import accis_single_idf as accis

import besos.eppy_funcs as ef
from besos.errors import InstallationError

# Using eppy

# iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
#
# fname = 'TestModel_onlyGeometryForVRFsystem_2zones_CalcVent_V940.idf'
#
# IDF.setiddname(iddfile)
# idf = IDF(fname)

# Using besos
fname = 'TestModel_onlyGeometryForVRFsystem_2zones_CalcVent_V940.idf'

try:
    idf = ef.get_building(fname)
except InstallationError:
    from accim.utils import amend_idf_version_from_dsb
    amend_idf_version_from_dsb(file_path=fname)
    idf = ef.get_building(fname)



# idf.idfobjects['zone']

# Using class structure

# version = f'{idf.idd_version[0]}.{idf.idd_version[1]}'

adaptive_idf = accis.addAccis(
    idf=idf,
    ScriptType='vrf_mm',
    SupplyAirTempInputMethod='temperature difference',
    Output_keep_existing=False,
    Output_type='standard',
    Output_freqs=['hourly'],
    EnergyPlus_version='9.4',
    TempCtrl='temperature',
    Output_gen_dataframe=True,

)



adaptive_idf.modifyAccis(
    ComfStand=1,
    CAT=3,
    ComfMod=3,
    # SetpointAcc=1000,
    HVACmode=2,
    VentCtrl=0,
    CoolSeasonStart='01/02',
    CoolSeasonEnd='01/03'
    # VSToffset=0,
    # MinOToffset=50,
    # MaxWindSpeed=50
)

# adaptive_idf_mod.idfobjects['energymanagementsystem:program']
# idf.idfobjects['energymanagementsystem:program']
adaptive_idf.SetInputData
adaptive_idf.SetVOFinputData
adaptive_idf.SetAST

# idf.savecopy('z_modified_to_adaptive_3.idf')

##

idf.idfobjects['output:variable']